
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

/*
 * asn_null.c - BER encode, decode, print and free routines for the 
 *              ASN.1 NULL type.
 *
 * MS 92
 * Copyright (C) 1992 Michael Sample and the University of British Columbia
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * $Header$
 * $Log$
 */

#include "asn_conf.h"
#include "asn_len.h"
#include "asn_tag.h"
#include "asn_null.h"


/*
 * encodes universal TAG LENGTH and Contents of and ASN.1 NULL
 */
AsnLen
BEncAsnNull PARAMS((b, data),
BUF_TYPE     b _AND_
AsnNull* data)
{
    AsnLen len;

    len =  BEncAsnNullContent(b, data);
    len += BEncDefLen(b, len);
    len += BEncTag1(b, UNIV, PRIM, NULLTYPE_TAG_CODE);
    return(len);
}  /* BEncAsnNull */


/* 
 * decodes universal TAG LENGTH and Contents of and ASN.1 NULL
 */
void
BDecAsnNull PARAMS((b, result, bytesDecoded, env),
BUF_TYPE b _AND_
AsnNull* result _AND_
AsnLen* bytesDecoded _AND_
jmp_buf env)
{
    AsnTag tag;
    AsnLen elmtLen;

    if ( (tag =BDecTag(b, bytesDecoded, env)) !=
        MAKE_TAG_ID(UNIV, PRIM, NULLTYPE_TAG_CODE))
    {
         Asn1Error("BDecAsnNull: ERROR wrong tag on NULL.\n");
         longjmp(env, -40);
    }

    elmtLen = BDecLen (b, bytesDecoded, env);
    BDecAsnNullContent( b, tag, elmtLen, result, bytesDecoded, env);

}  /* BDecAsnNull */



void
BDecAsnNullContent PARAMS((b, tagId, len, result, bytesDecoded, env),
BUF_TYPE b _AND_
AsnTag tagId _AND_
AsnLen len _AND_
AsnNull* result _AND_
AsnLen* bytesDecoded _AND_
jmp_buf env)
{
    if ( len != 0)
    {
        Asn1Error("BDecAsnNullContent: ERROR - NULL type's len must be 0\n");
        longjmp(env, -17);
    }
}  /* BDecAsnNullContent */

/*
 * Prints the NULL value to the given FILE* in Value Notation. 
 * ignores the indent.
 */
void
PrintAsnNull PARAMS((f,v, indent),
FILE* f _AND_
AsnNull* v _AND_
unsigned short int indent)
{
    fprintf(f,"NULL");
}
