
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

/*
 * nibble_alloc.c - fast mem allocation for decoded values
 *
 * MS Dec 31/91
 *
 * Copyright (C) 1992 Michael Sample and the University of British Columbia
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * $Header$
 * $Log$
 */

#include <memory.h>
#include "asn_conf.h"
#include "asn_nibl.h"

NibbleMem* nmG = NULL;

void
InitNibbleMem PARAMS((initialSize, incrementSize),
unsigned long int initialSize _AND_
unsigned long int incrementSize)
{
    NibbleMem* nm;

    nm = (NibbleMem*) RescIR_Zmalloc(IR_FILE, sizeof(NibbleMem));
    nm->incrementSize = incrementSize;

    nm->currNibbleBuf = nm->firstNibbleBuf =
        (NibbleBuf*)RescIR_Zmalloc(IR_FILE, sizeof(NibbleBuf));
    nm->firstNibbleBuf->curr = nm->firstNibbleBuf->start =
        (char*) RescIR_Zmalloc(IR_FILE, initialSize);
    nm->firstNibbleBuf->end = nm->firstNibbleBuf->start + initialSize;
    nm->firstNibbleBuf->next = NULL;
    memset((char *)(nm->currNibbleBuf->start), (int)0, (int)initialSize);

    nmG = nm;/* set global */

}  /* InitNibbleAlloc */


/*
 * alloc new nibble buf, link in, reset to curr nibble buf
 */
void
ServiceNibbleFault PARAMS( (size),
unsigned long size)
{
    NibbleMem* nm;
    unsigned long newBufSize;

    nm = nmG;

    if (size > nm->incrementSize)
        newBufSize = size;
    else
        newBufSize = nm->incrementSize;
    
    nm->currNibbleBuf->next = (NibbleBuf*) RescIR_Zmalloc(IR_FILE, sizeof(NibbleBuf));
    nm->currNibbleBuf = nm->currNibbleBuf->next;
    nm->currNibbleBuf->curr = nm->currNibbleBuf->start
        = (char*) RescIR_Zmalloc(IR_FILE, newBufSize);
    nm->currNibbleBuf->end = nm->currNibbleBuf->start + newBufSize;
    nm->currNibbleBuf->next = NULL;
    memset((char *)(nm->currNibbleBuf->start), (int)0, (int)newBufSize);
} /* serviceNibbleFault */



/*
 * returns requested space filled with zeros
 */
void*
NibbleAlloc PARAMS( (size),
unsigned long size)
{
    NibbleMem* nm;
    char* retVal;
    unsigned long ndiff;

    nm = nmG;

    if ((unsigned long)(nm->currNibbleBuf->end - nm->currNibbleBuf->curr) < size)
         ServiceNibbleFault(size);
 
    retVal = nm->currNibbleBuf->curr;

    /*
     * maintain word alignment
     */
    ndiff = size % sizeof(long);
    if (ndiff != 0)
	{
        nm->currNibbleBuf->curr += size + sizeof(long) - ndiff;

        /* 
         * this is a fix from Terry Sullivan <FCLTPS@nervm.nerdc.ufl.edu>
         * 
         * makes sure curr does not go past the end ptr
         */
        if (nm->currNibbleBuf->curr > nm->currNibbleBuf->end)
            nm->currNibbleBuf->curr = nm->currNibbleBuf->end;
	}
    else
        nm->currNibbleBuf->curr += size;

    return(retVal);
}  /* NibbleAlloc */



/*
 * frees all nibble buffers except the first,
 * resets the first to empty and zero's it
 */
void
ResetNibbleMem()
{
    NibbleMem* nm;
/*  int i;	    * UNUSED */
    NibbleBuf* tmp;
    NibbleBuf* nextTmp;

    nm = nmG;

    /*
     * reset first nibble buf
     */
    memset(nm->firstNibbleBuf->start, 0,
          nm->firstNibbleBuf->curr - nm->firstNibbleBuf->start);

    nm->firstNibbleBuf->curr = nm->firstNibbleBuf->start;

    /*
     * free incrementally added nibble bufs
     */
    for (tmp = nm->firstNibbleBuf->next; tmp != NULL; )
    {
        RescIR_Free(IR_FILE, (CHAR *)tmp->start);
        nextTmp = tmp->next;
        RescIR_Free(IR_FILE, (CHAR *)tmp);
        tmp = nextTmp;
    }

} /* ResetNibbleMem */


/*
 * frees all nibble buffers, closing this
 * NibbleMem completely
 */
void
ShutdownNibbleMem()
{
    NibbleMem* nm;
/*  int i;		* UNUSED */
    NibbleBuf* tmp;
    NibbleBuf* nextTmp;

    nm = nmG;
    nmG = NULL;
    /*
     * free nibble bufs
     */
    for (tmp = nm->firstNibbleBuf; tmp != NULL; )
    {
        RescIR_Free(IR_FILE, (CHAR *)tmp->start);
        nextTmp = tmp->next;
        RescIR_Free(IR_FILE, (CHAR *)tmp);
        tmp = nextTmp;
    }

    RescIR_Free(IR_FILE, (CHAR *)nm);
} /* ShutdownNibbleMem */
