/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log: user.c,v $
 * Revision 1.1  1993/11/19  21:25:27  irorigin
 * Initial revision
 *
 ************************************************************************/

#include "cfg.h"

/*
 * This driver is used to read in user information and
 * create records in the format required by the IR ORIGIN Table Management CSU.
 * These records are output to the file indicated.
 *
 * The input file is prompted for by the targets routine.
 * The format of the input data consists of:
 *      output file name
 *      list of users
 *
 *      - each user is preceded by USER 
 *      - each user element is preceded (on the same line by one of the 
 *      following:
 *
 *      USERNAME:
 *      PASSWORD:       
 *      PRIVILEGE:       
 *      PREFLANG:        
 *      ACCOUNTCREDITINFO:       
 *      TARGETINFOFILE:  
 *
 */


#define USER_TABLE_SIZE	7

static CFG_XREF userTable[USER_TABLE_SIZE] = {  
			{"USER:", CFG_USER},
			{"USERNAME:", CFG_USER_NAME},
			{"PASSWORD:", CFG_PASSWORD},
			{"PRIVILEGE:", CFG_PRIVILEGE},
			{"PREFLANG:", CFG_PREF_LANG},
			{"ACCOUNTCREDITINFO:", CFG_CREDIT},
			{"TARGETINFOFILE:", CFG_TRGT_FILE}};

static CFG_RECORD record;
static CFG_RECORD_FIELD field;

static CHAR rec[CFG_MAX_RECORD_SIZE];
static CHAR buf[CFG_MAX_RECORD_SIZE];

static CHAR fileName[CFG_MAX_FILE_NAME_SIZE];

void main()
{
	FILE    *i_fp, *o_fp;
	CHAR    *data;
	CHAR    *dataStart;
	SHORT	tag;
	BOOLEAN	user;

	printf("Enter user profile file name:\n");
	gets(fileName);
	i_fp = fopen(fileName, "r");
	if (i_fp == NULL)
	{
		printf("Error opening input file %s\n", fileName);
		exit(2);
	}

	/* get output file name */
	fgets(buf, CFG_MAX_RECORD_SIZE, i_fp);
	data = Cfg_getData(buf);
	printf("Output file is %s\n", data);

	o_fp = fopen(data, "a+");
	if (o_fp == NULL)
	{
		printf("Error opening output file %s\n", fileName);
		fclose(i_fp);
		exit(2);
	}

	Cfg_initRecord(&record, CFG_USER_INFO, CFG_NULL_KEY_S, rec);
	user = FALSE;
	fgets(buf, CFG_MAX_RECORD_SIZE, i_fp); 
	while (!feof(i_fp))
	{
		tag = Cfg_getTag(userTable, USER_TABLE_SIZE, buf, &dataStart); 
		data = Cfg_getData(dataStart);
		switch (tag)
		{
			case CFG_BLANK_LINE:
				break;

			case CFG_NULL_TAG:
				printf("invalid keyword in line:\n");
				printf("\t%s\n", buf);
				fclose(o_fp);
				fclose(i_fp);
				exit(-1);
				break;

			case CFG_USER:
				if (user)
				{
					Cfg_writeRecord(o_fp, &record);
					Cfg_initRecord(&record, CFG_USER_INFO, CFG_NULL_KEY_S, rec);
				}
				user = TRUE;
				break;

			case CFG_USER_NAME:
				if (user && data)
				{
					Cfg_setRecordKey(&record, data);
					Cfg_initField(&field, tag, data);
					Cfg_appendField(&record, &field);
					printf("User: %s\n", data);
				}
				break;

			default:
				if (user && data)
				{
					Cfg_initField(&field, tag, data);
					Cfg_appendField(&record, &field);
				}
				break;
		}
		fgets(buf, CFG_MAX_RECORD_SIZE, i_fp);
	}

	if (user)
	{
		Cfg_writeRecord(o_fp, &record);
	}

	fclose(o_fp);
	fclose(i_fp);
	exit(1);

}
