
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: CscF_Close
 *
 * DESCRIPTION:
 *	This unit is used to perform a close on an open socket.
 *
 * CONTENTS:
 *	CsCf_CloseSocket
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#include <ir_pli.h>
#include <ir_cs.h>
#include <resc.h>

/****************************************************************************
 * FUNCTION: CsCf_CloseSocket
 *
 * PURPOSE:
 *	This function will get the socket information from the association
 *	table and close the socket. If the socket closes successfully then
 *	it will remove the association entry from the association table list.
 *
 * INPUT:
 *	assocId			- association identifier to close.
 *
 * OUTPUT:
 *	This function returns an IR_STATUS which returns a non-zero
 *	value on error. 
 *
 *
 * NOTES:
 *	This function should only be called from the comm event unit.
 *
 ***************************************************************************/

IR_STATUS	CsCf_CloseSocket( IR_ASSOC_ID assocId )
{
	CS_SOCK_ITEM 		*sockItem;
	SHORT			errorCode;
	static BOOLEAN		firstTime=TRUE;
#ifdef WINSOCKET
	struct linger		lingStruct;
#endif

	/* Get the association's state information. */
	sockItem = CsCf_AssocGet(assocId);
	if (!sockItem)
	{
		CS_LOG_PRINTV(IR_FILE, "AssocGet failed: assocId = %d\n",
		    assocId);
		return(IR_SUCCESS);
	}

	errorCode = 0;
	if (sockItem->socketFd != IR_UNKNOWN_ID_TYPE)
	{
		CS_LOG_PRINTV(IR_FILE, "Closing socket: %d\n",
		    sockItem->socketFd);
#ifdef WINSOCKET
		lingStruct.l_onoff = 1;
		lingStruct.l_linger = 0;
		(void)setsockopt(sockItem->socketFd,
		    (int)SOL_SOCKET, (int)SO_LINGER, 
		    (char *)&lingStruct, sizeof(struct linger));
		if (closesocket(sockItem->socketFd) != 0)
		{
			errorCode = WSAGetLastError();
		}
#endif
#ifdef BSDSOCKET
		if (close(sockItem->socketFd) != 0)
		{
			errorCode = errno;
		}
#endif
#ifdef WINPATHWAY
		if (sock_close(sockItem->socketFd) != 0)
		{
			errorCode = get_errno();
		}
#endif
	}
	else if (sockItem->handle != IR_UNKNOWN_ID_TYPE)
	{
#ifdef WINSOCKET
		WSACancelAsyncRequest(sockItem->handle);
#endif
	}

	if (errorCode != 0) 
	{
		/* 
		 * If close failed the first time then try once more 
		 * to close it. If that fails then assume it is closed.
		 */
		CS_LOG_PRINTV(IR_FILE, "Error closing socket. Error = %d\n",
		    errorCode);
		if (firstTime == TRUE)
		{
		    	firstTime = FALSE;
		    	CsCf_CloseSocket( assocId );
			firstTime = TRUE;
		}
	}

	/* Assume socket was deleted if we have made it this far. */
	CsCf_AssocDel(assocId);
	return (IR_SUCCESS);
}
