
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: CF_Notify
 *
 * DESCRIPTION:
 *	This unit contains functions used by the control function to post events *	to a window for communication events.
 *
 * CONTENTS:
 *	CsCf_Notify
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#include <ir_cs.h>

/****************************************************************************
 * FUNCTION: CsCf_Notify
 *
 * PURPOSE:
 *	This function will force an event into the window system event
 *	queue. If a window system is not used then this function will
 *	just post a message into the simulated event queue.
 *
 * INPUT:
 *	assocId		- window and association identifier
 *	event		- event to post.
 *
 *
 * OUTPUT:
 *	IR_STATUS	- error if invalid assocId.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS CsCf_Notify(IR_ASSOC_ID assocId, CS_SOCK_EVENT event)
{
	if (assocId == IR_UNKNOWN_ID_TYPE)
	{
		return (IR_FAIL);
	}

	CS_LOG_PRINTV(IR_FILE, "Posting event %d\n", event);
#ifdef XVT_MSDOS
	/*
	 * Post the message to the event queue.
	 */
	PostMessage((HWND)assocId, event, 0, 0);
#else
	/*
	 * Post the message to a simulated event queue.
 	 */
	Resc_SimPostMessage(assocId, event);
#endif
}

