
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: IRC Release
 *
 * DESCRIPTION:
 *	This interface function is used to terminate a connection to
 *	a remote target in an orderly fashion.
 *
 * CONTENTS:
 *	IRC_Release
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#include <ir_pli.h>
#include <ir_cs.h>
#include <resc.h>


/****************************************************************************
 * FUNCTION: IRC_Release
 *
 * PURPOSE:
 *	This function is used to perform an orderly release of an association.
 *	Because their is no orderly release associated with an association
 *	at this time in a TCP environment then this function will perform 
 *	something very similar to abort.
 *
 * INPUT:
 *	assocId		The association to release.
 *
 * OUTPUT:
 *	IR_STATUS
 *
 *
 * NOTES:
 *
 ***************************************************************************/

IR_STATUS	IRC_Release( IR_ASSOC_ID assocId )
{
	CS_SOCK_ITEM 		*sockItem;
#ifdef WINSOCKET
	struct linger		lingStruct;
#endif


	if (!CsCf_IsIRCInitialized())
	{
		CsCf_ReportError(IR_FILE, CS_ERR_IRC_NOT_INIT, NO_STRING,
		    NO_DATA_VALUE);
		return(IR_FAIL);
	}
	/* Get the association's state information. */
	sockItem = CsCf_AssocGet(assocId);
	if (!sockItem)
	{
		CS_LOG_PRINTS(IR_FILE, "AssocStateGet failed\n");
		CS_LOG_PRINTS(IR_FILE, "We will fake the abort\n");
		/* Fake it and return success */
		return(IR_SUCCESS);
	}
	else
	{
		sockItem->state = CS_SOCK_ST_ABORT;
		CsCf_Notify(assocId, (CS_SOCK_EVENT)CS_SOCK_EV_USER_RELEASE);
		return (IR_SUCCESS);
	}
}
