
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/***************************************************************************
 * UNIT: send.c 
 *
 * DESCRIPTION: This unit contains the functionality to support
 * 	the IRK_SendRequest primitive provided
 *	by the Origin Kernel IRASE and the functionality to process\
 * 	a response. 
 *
 * CONTENTS: IRK_SendRequest
 *	     OkAse_ReceiveResponse
 *
 * NOTES: 
 *	Nothing of note.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
***************************************************************************/
#include <stdio.h>
#include <apdu.h>
#include <asn_tool.h>
#include <ir.h>
#include <ir_ali.h>
#include <ir_pli.h>
#include <resc.h>
#include <ok/ok.h>

#ifdef FILENAME
#undef FILENAME
#endif
#define FILENAME "OKIRASE send.c"

/****************************************************************************
 * FUNCTION: IRK_SendRequest
 *
 * PURPOSE: Determine whether a request is valid for the given association,
 *	and invoke the PLI primitive to send the request.
 *
 * INPUT: 
 *    	IRK_REQ_SERVICE_TYPE 	event
 *	IR_ASSOC_ID		association identifier
 *	IRK_SEND_REQUEST_REQ	pointer to request structure
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS IRK_SendRequest(
    	IRK_REQ_SERVICE_TYPE 	event,
	IR_ASSOC_ID		assocId,
	IRK_SEND_REQUEST_REQ	*requestReq)
{
	IR_STATUS	status;
	SHORT		eventId;
	IRC_P_DATA_SEND	pData;

	if (!OkCf_IsIRKInitialized())
	{
		return(IR_FAIL);
	}

	/* Determine event type. */
	switch (event)
	{
	case IRK_INIT_REQ_TYPE: 
		eventId = OK_INIT_REQ;
		break;
	case IRK_SEARCH_REQ_TYPE: 
		eventId = OK_SEARCH_REQ;
		break;
	case IRK_PRESENT_REQ_TYPE: 
		eventId = OK_PRESENT_REQ;
		break;
	case IRK_DELETE_REQ_TYPE: 
		eventId = OK_DELETE_REQ;
		break;
	case IRK_TRIG_REQ_TYPE: 
		eventId = OK_TRIG_CNTRL_REQ;
		break;
	case IRK_RES_RPT_REQ_TYPE: 
		eventId = OK_RSRC_REP_REQ;
		break;
	case IRK_RELEASE_REQ_TYPE: 
		eventId = OK_RELEASE_REQ;
		break;
	case IRK_ABORT_REQ_TYPE: 
		eventId = OK_I_ABORT_REQ;
		break;
	case IRK_ACCESS_RES_TYPE: 
		eventId = OK_ACCESS_CNTRL_RES;
		break;
	case IRK_RESRC_CNTL_RES_TYPE:
		eventId = OK_RSRC_CNTRL_RES;
		break;
	default: 
		eventId = OK_INVALID;
	}	
	
	/* Service request internally, or verify validity of request
	   for the association, and if valid invoke PLI primitive to send. 
	   If request sent successfully, update the association state table. */
	if (eventId == OK_INVALID)
	{
		/* Error - invalid event */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING, IRK_INVALID_EVENT,
		    IR_FILE);
		return (IR_FAIL);
	}
	else if (eventId == OK_I_ABORT_REQ)
	{
		status = OkAcse_AAbort(assocId);
		if (status == IR_FAIL)
		{
			return (IR_FAIL);
		}
#if 0
		status = OkAse_UpdateState(assocId, eventId);
#endif
	}
	else if (eventId == OK_RELEASE_REQ)
	{
		status = OkAse_CheckEvent(assocId, eventId);
		if (status == IR_FAIL)
		{
			return (IR_FAIL);
		}
		status = OkAcse_ARelease(assocId);
		if (status == IR_FAIL)
		{
			return (IR_FAIL);
		}
#if 0
		status = OkAse_UpdateState(assocId, eventId);
#endif
	}
	else
	{
		status = OkAse_CheckEvent(assocId, eventId);
		if (status == IR_FAIL)
		{
			return (IR_FAIL);
		}
		/* Send the apdu */
		pData.apdu = &requestReq->irApdu;
		status = IRC_P_Data(assocId, (IRC_P_DATA_SEND *)&pData);
		if (status == IR_FAIL)
		{
			OkCf_UpdateErrorReport(IR_ERROR_WARNING, 
			    IRK_PLI_ERROR, IR_FILE);
			return (IR_FAIL);
		}
		status = OkAse_UpdateState(assocId, eventId);
	}
	return (status);
}

/****************************************************************************
 * FUNCTION: OkAse_ReceiveResponse
 *
 * PURPOSE: To update the state table as a result of a communication event.
 *
 * INPUT: 
 *	IR_ASSOC_ID		assocId
 *	IR_APDU			*apdu
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS OkAse_ReceiveResponse(
	IR_ASSOC_ID		assocId,
	APDU			*apdu)
{
	IR_STATUS 	status;
	SHORT		eventId;
	BIT_STRING	services;
	CHAR		*srvMask;
	CHAR		noMask[2];
	USHORT		byteCnt;
	USHORT		bitCnt;
	SHORT		stat;

	/* Determine APDU type. */
	switch (apdu->choiceId)
	{
	case APDU_INITRESP: 
		RESC_ASN_NO_MALLOC(&services);
		if (apdu->a.initResp->result == TRUE)
		{
			eventId = OK_INIT_ACC_RES;
			status = OkAse_CheckEvent(assocId, eventId);
			if (status == IR_FAIL)
			{
				CS_LOG_PRINTV(IR_FILE, "Event Update failed %d\n", eventId);
				return (IR_FAIL);
			}

			stat = ASN_BitsGet(&srvMask, &byteCnt, &bitCnt, 
			    &(apdu->a.initResp->options));
			if (stat != 0)
			{
				OkCf_UpdateErrorReport(IR_ERROR_FATAL, 
				     IRK_MEMORY_ERROR, IR_FILE);
				return (IR_FAIL);
			}
			RescASN_AddBS(IR_FILE, &services, srvMask, 
			    (USHORT)(byteCnt * 8 + bitCnt));
			status = OkAse_InitAssoc(assocId, &services);	
			if (status == IR_FAIL)
			{
				CS_LOG_PRINTS(IR_FILE, "InitAssoc failed\n");
				return (IR_FAIL);
			}
		}
		else
		{
			eventId = OK_INIT_REJ_RES;
			status = OkAse_UpdateState(assocId, eventId);
			if (status == IR_FAIL)
			{
				return (IR_FAIL);
			}
			noMask[0] = (CHAR)0x00;
			RescASN_AddBS(IR_FILE, &services, noMask, (USHORT)1);
			status = OkAse_InitAssoc(assocId, &services);	
			if (status == IR_FAIL)
			{
				return (IR_FAIL);
			}
		}
		break;
	case APDU_SEARCHRESP: 
		eventId = OK_SEARCH_RES;
		break;
	case APDU_PRESENTRESP: 
		eventId = OK_PRESENT_RES;
		break;
	case APDU_DELETERESULTSETRESP: 
		eventId = OK_DELETE_RES;
		break;
	case APDU_ACCESSCONTROLREQ: 
		eventId = OK_ACCESS_CNTRL_REQ;
		break;
	case APDU_RESOURCECONTROLREQ: 
		if (apdu->a.resourceControlReq->responseRequired
		    == TRUE)
		{
			eventId = OK_RSRC_R_CNTRL_REQ;
		}
		else
		{
			eventId = OK_RSRC_NR_CNTRL_REQ;
		}
		break;
	case APDU_RESOURCEREPORTRESP: 
		eventId = OK_RSRC_REP_RES;
		break;
	default: 
		eventId = OK_INVALID;
	}	
	
	/* Process event */
	if (eventId == OK_INVALID)
	{
		/* Error - invalid event */
		OkAcse_AAbort(assocId);
		OkCf_UpdateErrorReport(IR_ERROR_WARNING, IRK_PROTOCOL_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}
	else if(eventId != OK_INIT_REJ_RES) 
	{
		status = OkAse_UpdateState(assocId, eventId);
	}
	return (status);
}
