

/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpCf_IRO
 *
 * DESCRIPTION:
 *	This unit contains functions used to save and destroy data
 *	that is passed to the user application.
 *
 *	The IRO_ interface is designed such that any data that is 
 *	passed to the user application is only available to the user
 *	application until the next time that data is returned to the
 *	user application. This unit contains functions that will be
 *	used to record the data and the destroy function which will
 *	be called later.
 *
 * CONTENTS:
 *	OpCf_IROMark
 *	OpCf_IROFree
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <op/ir_op.h>
#include <resc.h>

#ifdef FILENAME
#undef FILENAME
#endif
#define FILENAME "OPCF cf_util"

static void (*fnPtr)(void *d) = NULL;
static void *dataPtr = NULL;

/****************************************************************************
 * FUNCTION:  OpCf_IROMark
 *
 * PURPOSE:
 *	This function is used to store the free function pointer and the
 *	pointer to the data to be freed. If the previous pointers exist 
 *	then the free function will be invoked before the new data is 
 *	saved.
 *
 * INPUT:
 *	pointer to function returning void
 *	pointer to void data
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/
void OpCf_IROMark(void (*markFn)(void *d), void *data)
{
	
	if ((fnPtr != NULL) && (dataPtr != NULL))
	{
		OpCf_IROFree();
	}
	
	fnPtr = markFn;
	dataPtr = data;
}

/****************************************************************************
 * FUNCTION:  OpCf_IROFree
 *
 * PURPOSE:
 *	This function is used to free data previously sent to the user
 *	application.
 *
 * INPUT:
 *	none.
 *
 * OUTPUT:
 *	none.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void OpCf_IROFree()
{
	if ((fnPtr != NULL) && (dataPtr != NULL))
	{
		(*fnPtr)(dataPtr);
	}
	fnPtr = NULL;
	dataPtr = NULL;
}

