

/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpCf_Utility
 *
 * DESCRIPTION:
 *	This unit contains utility functions required to assist the 
 *	origin process cscs when apdus are being generated.
 *
 * CONTENTS:
 *	OpCf_GetReferenceId
 *	OpCf_CheckReferenceId
 *	OpCf_GetString
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <string.h>
#include <ir_api.h>
#include <op/ir_op.h>
#include <resc.h>
#include <asn_tool.h>

#ifdef FILENAME
#undef FILENAME
#endif
#define FILENAME "OPCF cf_util"

/****************************************************************************
 * FUNCTION:  OpCf_GetString
 *
 * PURPOSE:
 *	This function is used to convert an ASN_OCT to a null terminated 
 *	string.
 *
 * INPUT:
 *	*AsnOcts	pointer to source asn_oct
 *	**CHAR		pointer to pointer to destination string
 *
 * OUTPUT:
 *	IR_STATUS	
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS OpCf_GetString(AsnOcts *src, CHAR **dest)
{
	CHAR 	*buf;
	CHAR 	*tmpPtr;
	ULONG	count;
	SHORT	stat;

	stat = ASN_OctsGet(&buf, &count, src); 
	if (stat != 0)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_MEMORY_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}
	tmpPtr = (CHAR *)RescIR_Malloc(IR_FILE, (ULONG)(count + 1));
	if (tmpPtr == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_MEMORY_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}
	(void)memcpy((void *)tmpPtr, (void *)buf, (int)count);
	tmpPtr[(int)count] = (char)'/0';
	*dest = tmpPtr;
	return (IR_SUCCESS);
}
