
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpPm_Util
 *
 * DESCRIPTION:
 *	This unit contains utility functions for Presentation Processing CSC
 *
 * CONTENTS:
 *   OpPm_ElemSetNameToSnacc
 *   OpPm_ElemSetListToSnacc
 *
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <apdu.h>
#include <string.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <op/ir_op.h>
#include <asn_tool.h>
#include <ir_oid.h>
#include <resclist.h>

void OpPm_ElemSetNameToSnacc(VISIBLE_STRING *irElem, 
    ElementSetNames *snaccElem);
void OpPm_ElemSetListToSnacc(IR_LIST *irElemList, 
    ElementSetNames *snaccElemList);

/****************************************************************************
 * FUNCTION:  OpPm_ElemSetNameToSnacc
 *
 * PURPOSE:
 *	This function is used to copy an element set name from 
 *	IR format to Snacc format. This function is used only to convert
 *	the generic element set name.
 *
 * INPUT:
 *	ptr IR generic element set name (Visible String)
 *	ptr Snacc element set name structure. 
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/
void OpPm_ElemSetNameToSnacc(VISIBLE_STRING *irElem, 
    ElementSetNames *snaccElem)
{
	snaccElem->choiceId = ELEMENTSETNAMES_GENERIC;

	if ((snaccElem->a.generic = ASN_OctsCreate()) == (AsnOcts *)NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Error Converting element set name\n");
	}
	else
	{
		RescASN_IRToSnaccVS(IR_FILE, snaccElem->a.generic, irElem);
	}
}

/****************************************************************************
 * FUNCTION:  OpPm_ElemSetListToSnacc
 *
 * PURPOSE:
 *	This function is used to copy an element set list from 
 *	IR format to Snacc format. This function is used only to convert
 *	the specific element set name list.
 *
 * INPUT:
 *	ptr IR specifc element set name list ( IR_LIST ) 
 *	ptr Snacc element set name structure. 
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/
void OpPm_ElemSetListToSnacc(IR_LIST *irElemList, 
    ElementSetNames *snaccElemList)
{
	IR_SPECIFIC_ELEM_SET_NAME 	*irElem;
	ElementSetNamesSeqOfSeq		*snaccElem;
	BOOLEAN				done;

	snaccElemList->choiceId = ELEMENTSETNAMES_DATABASESPECIFIC;

	if ((snaccElemList->a.databaseSpecific = 
	    ASN_ListCreate(sizeof(ElementSetNamesSeqOfSeq)))
	    == (AsnList *)NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Cannot create element set list\n");
	}
	else
	{
		/* 
		 * For each element in the ir list copy it to the 
	 	 * Snacc list.
		 */
		done = FALSE;
		RESC_LIST_SET_FIRST(irElemList);

		while (!done)
		{
			irElem = RESC_LIST_GET(irElemList);
			if ((snaccElem = 
			    (ElementSetNamesSeqOfSeq *)RescIR_Zmalloc(IR_FILE,
			    sizeof(ElementSetNamesSeqOfSeq))) !=
			    (ElementSetNamesSeqOfSeq *)NULL)
			{
				/* Copy ir to snacc */
				RescASN_IRToSnaccVS(IR_FILE, &snaccElem->databaseName, 
			    	    &irElem->dbName);
				RescASN_IRToSnaccVS(IR_FILE, &snaccElem->elementSetName,
				    &irElem->elemSetName);
				
			}
			/* Get next IR Element */
			if (RESC_LIST_IS_LAST(irElemList))
			{
				done = TRUE;
			}
			else
			{
				RESC_LIST_SET_NEXT(irElemList);
			}
		}
		
	}
}
