
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: Resc ASN Utilities
 *
 * DESCRIPTION:
 *	This file contains functions used to create and destroy basic
 *	IR ASN.1 types.
 *
 * CONTENTS:
 *	RescASN_CreateVS	- create visible string
 *	RescASN_CreateOS	- create octet string
 *	RescASN_CreateBS	- create bit string
 *	RescASN_CreateOI	- create object identifier
 *	RescASN_CreateSI	- create short int
 *	RescASN_CreateLI	- create long int
 *	RescASN_DestroyVS	- create visible string
 *	RescASN_DestroyOS	- create octet string
 *	RescASN_DestroyBS	- create bit string
 *	RescASN_DestroyOI	- create object identifier
 *	RescASN_DestroySI	- create short int
 *	RescASN_DestroyLI	- create long int
 *	RescASN_AddVS		- add the visible string to a vs struct.
 *	RescASN_AddOS		- add the octet  string to a vs struct.
 *	RescASN_AddBS		- add the bit string to a vs struct.
 *	RescASN_AddOI		- add the object identifier to a vs struct.
 *	RescASN_IRToSnaccVS	- Copy IR VS to SNACC VS
 *	RescASN_IRToSnaccOS	- Copy IR OS to SNACC OS
 *	RescASN_IRToSnaccBS	- Copy IR BS to SNACC BS
 *	RescASN_IRToSnaccOI	- Copy IR OI to SNACC OI
 *	RescASN_SnaccToIRVS	- Copy Snacc VS to IR VS
 *	RescASN_SnaccToIRBS	- Copy Snacc BS to IR BS
 *	RescASN_SnaccToIROI	- Copy Snacc OI to IR OI
 *	RescASN_CloneVS		- Clone a visible string.
 *	RescASN_CompareVS	- Compare to visible strings
 *	RescASN_CmpOIToStrOI	- Compare OBJECT_IDENTIFIER to string
 *
 * NOTES:
 *
 *
 *
/***************************************************************************/

#include <ir.h>
#include <resclist.h>
#include <resc.h>
#include <string.h>
#include <memory.h>
#include <asn_tool.h>
#include <asn_incl.h>

VISIBLE_STRING	*RescASN_CreateVS(CHAR *srcFile)
{
	VISIBLE_STRING	*vs;

	vs = (VISIBLE_STRING *)RescIR_Malloc(srcFile, 
	    sizeof(VISIBLE_STRING));

	RESC_ASN_MALLOCED(vs);
	vs->len = 0;

	return(vs);
}
OCTET_STRING *RescASN_CreateOS(CHAR *srcFile)
{
	OCTET_STRING *os;

	os = (OCTET_STRING *)RescIR_Malloc(srcFile, 
	    sizeof(OCTET_STRING));

	RESC_ASN_MALLOCED(os);
	os->len = 0;

	return(os);
}
BIT_STRING *RescASN_CreateBS(CHAR *srcFile)
{
	BIT_STRING *bs;

	bs = (BIT_STRING *)RescIR_Malloc(srcFile, 
	    sizeof(BIT_STRING));

	RESC_ASN_MALLOCED(bs);
	bs->len = 0;

	return(bs);
}
OBJECT_IDENTIFIER *RescASN_CreateOI(CHAR *srcFile)
{
	OBJECT_IDENTIFIER *oi;

	oi = (OBJECT_IDENTIFIER *)RescIR_Malloc(srcFile, 
	    sizeof(OBJECT_IDENTIFIER));

	RESC_ASN_MALLOCED(oi);
	oi->len = 0;

	return(oi);
}

SHORT	 *RescASN_CreateSI(CHAR *srcFile)
{
	SHORT	*si;

	si = (SHORT *)RescIR_Malloc(srcFile, sizeof(SHORT));

	return(si);
}
LONG *RescASN_CreateLI(CHAR *srcFile)
{
	LONG *li;

	li = (LONG *)RescIR_Malloc(srcFile, sizeof(LONG));

	return(li);
}

void RescASN_DestroyVS(CHAR *srcFile, void *pData)
{
	VISIBLE_STRING *vs;

	vs = (VISIBLE_STRING *)pData;

	if ((vs->sPtr) && (vs->len > 0))
	{
		RescIR_Free(srcFile, (CHAR *)vs->sPtr);
	}

	if (RESC_ASN_IS_MALLOCED(vs))
	{

		RescIR_Free(srcFile, (CHAR *)vs);
	}
	else
	{
		RESC_ASN_INIT_VS(vs);
	}
}
	
void RescASN_DestroyOS(CHAR *srcFile, void *pData)
{
	OCTET_STRING *os;

	os = (OCTET_STRING *)pData;

	if ((os->sPtr) && (os->len > 0))
	{
		RescIR_Free(srcFile, (CHAR *)os->sPtr);
	}

	if (RESC_ASN_IS_MALLOCED(os))
	{
		RescIR_Free(srcFile, (CHAR *)os);
	}
	else
	{
		RESC_ASN_INIT_VS(os);
	}
}
	
void RescASN_DestroyBS(CHAR *srcFile, void *pData)
{
	BIT_STRING *bs;

	bs = (BIT_STRING *)pData;

	if ((bs->sPtr) && (bs->len > 0))
	{
		RescIR_Free(srcFile, (CHAR *)bs->sPtr);
	}

	if (RESC_ASN_IS_MALLOCED(bs))
	{
		RescIR_Free(srcFile, (CHAR *)bs);
	}
	else
	{
		RESC_ASN_INIT_VS(bs);
	}
}
	
void RescASN_DestroyOI(CHAR *srcFile, void *pData)
{
	OBJECT_IDENTIFIER *oi;

	oi = (OBJECT_IDENTIFIER *)pData;

	if ((oi->sPtr) && (oi->len > 0))
	{
		RescIR_Free(srcFile, (CHAR *)oi->sPtr);
	}
	
	if (RESC_ASN_IS_MALLOCED(oi))
	{
		RescIR_Free(srcFile, (CHAR *)oi);
	}
	else
	{
		RESC_ASN_INIT_VS(oi);
	}
}
	
void RescASN_DestroySI(CHAR *srcFile, void *pData)
{
	SHORT *si;

	si = (SHORT *)pData;

	if (si)
	{
		RescIR_Free(srcFile, (CHAR *)si);
	}
}
	
void RescASN_DestroyLI(CHAR *srcFile, void *pData)
{
	LONG  *li;

	li = (LONG  *)pData;

	if (li)
	{
		RescIR_Free(srcFile, (CHAR *)li);
	}
}
	
IR_STATUS	RescASN_AddVS(CHAR *srcFile, VISIBLE_STRING *vs, 
		    CHAR *str, USHORT len)
{
	if (len > 0)
	{
		vs->sPtr = RescIR_Malloc(srcFile, len);
		if (vs->sPtr == (CHAR *)NULL)
		{
			CS_LOG_PRINTS(srcFile, "Error mallocing visible string\n");
			return(IR_FAIL);
		}
		memcpy(vs->sPtr, str, len);
	}
	vs->len = len;
	return(IR_SUCCESS);
}

IR_STATUS	RescASN_AddOS(CHAR *srcFile, OCTET_STRING *os, 
		    CHAR *str, USHORT len)
{
	if (len > 0)
	{
		os->sPtr = RescIR_Malloc(srcFile, len);
		if (os->sPtr == (CHAR *)NULL)
		{
			CS_LOG_PRINTS(srcFile, "Error mallocing octet string\n");
			return(IR_FAIL);
		}
		memcpy(os->sPtr, str, len);
	}
	os->len = len;
	return(IR_SUCCESS);
}

IR_STATUS	RescASN_AddBS(CHAR *srcFile, BIT_STRING *bs, 
		    CHAR *str, USHORT len)
{
	USHORT	byteLen;

	if (len > 0)
	{
		byteLen = len / 8;
		if (len % 8)
		{
			byteLen++;
		}
		bs->sPtr = RescIR_Malloc(srcFile, byteLen);
		if (bs->sPtr == (CHAR *)NULL)
		{
			CS_LOG_PRINTS(srcFile, "Error mallocing bit string\n");
			return(IR_FAIL);
		}
		/* Save bit length */
		memcpy(bs->sPtr, str, byteLen);
	}
	bs->len = len;
	return(IR_SUCCESS);
}

IR_STATUS	RescASN_AddOI(CHAR *srcFile, OBJECT_IDENTIFIER *oi, 
		    CHAR *str, USHORT len)
{
	if (len > 0)
	{
		oi->sPtr = RescIR_Malloc(srcFile, len);
		if (oi->sPtr == (CHAR *)NULL)
		{
			CS_LOG_PRINTS(srcFile, "Error mallocing object identifier\n");
			return(IR_FAIL);
		}
		memcpy(oi->sPtr, str, len);
	}
	oi->len = len;
	return(IR_SUCCESS);
}

IR_STATUS RescASN_CopyVS(CHAR *srcFile, VISIBLE_STRING *dest, VISIBLE_STRING *src)
{

	if ((dest == (VISIBLE_STRING *)NULL) ||
	    (src == (VISIBLE_STRING *)NULL))
	{
		CS_LOG_PRINTS(srcFile, "Invalid pointers for copy\n");
		return(IR_FAIL);
	}

	dest->len = src->len;
	if (src->len > 0)
	{
		dest->sPtr = RescIR_Malloc(srcFile, src->len);
		memcpy(dest->sPtr, src->sPtr, src->len);
	}
	else
	{
		dest->sPtr = (CHAR *)NULL;
	}

	return(IR_SUCCESS);
}

IR_STATUS RescASN_CopyOS(CHAR *srcFile, OCTET_STRING *dest, OCTET_STRING *src)
{

	if ((dest == (OCTET_STRING *)NULL) ||
	    (src == (OCTET_STRING *)NULL))
	{
		CS_LOG_PRINTS(srcFile, "Invalid pointers for copy\n");
		return(IR_FAIL);
	}
	dest->len = src->len;
	if (src->len > 0)
	{
		dest->sPtr = RescIR_Malloc(srcFile, src->len);
		memcpy(dest->sPtr, src->sPtr, src->len);
	}
	else
	{
		dest->sPtr = (CHAR *)NULL;
	}

	return(IR_SUCCESS);
}

IR_STATUS RescASN_CopyBS(CHAR *srcFile, BIT_STRING *dest, BIT_STRING *src)
{
	USHORT	byteLen;

	if ((dest == (BIT_STRING *)NULL) ||
	    (src == (BIT_STRING *)NULL))
	{
		CS_LOG_PRINTS(srcFile, "Invalid pointers for copy\n");
		return(IR_FAIL);
	}

	dest->len = src->len;
	byteLen = src->len / 8;
	if (src->len % 8)
	{
		byteLen++;
	}
	if (src->len > 0)
	{
		dest->sPtr = RescIR_Malloc(srcFile, byteLen);
		memcpy(dest->sPtr, src->sPtr, byteLen);
	}
	else
	{
		dest->sPtr = (CHAR *)NULL;
	}

	return(IR_SUCCESS);
}

IR_STATUS RescASN_CopyOI(CHAR *srcFile, OBJECT_IDENTIFIER *dest, 
	     OBJECT_IDENTIFIER *src)
{

	if ((dest == (OBJECT_IDENTIFIER *)NULL) ||
	    (src == (OBJECT_IDENTIFIER *)NULL))
	{
		CS_LOG_PRINTS(srcFile, "Invalid pointers for copy\n");
		return(IR_FAIL);
	}
	dest->len = src->len;
	if (src->len > 0)
	{
		dest->sPtr = RescIR_Malloc(srcFile, src->len);
		memcpy(dest->sPtr, src->sPtr, src->len);
	}
	else
	{
		dest->sPtr = (CHAR *)NULL;
	}

	return(IR_SUCCESS);
}

IR_STATUS RescASN_IRToSnaccVS(CHAR *srcFile, AsnOcts *octs, VISIBLE_STRING *vs)
{
	if (vs->len == 0)
	{
		return(IR_SUCCESS);
	}
	if (ASN_OctsAdd(octs, vs->sPtr, (ULONG)vs->len) == 0)
	{
		return(IR_SUCCESS);
	}
	else
	{
		CS_LOG_PRINTS(srcFile, "Error converting IR to snacc VS\n");
		return(IR_FAIL);
	}
}

IR_STATUS RescASN_IRToSnaccOS(CHAR *srcFile, AsnOcts *octs, OCTET_STRING *os)
{
	if (os->len == 0)
	{
		return(IR_SUCCESS);
	}
	if (ASN_OctsAdd(octs, os->sPtr, (ULONG)os->len) == 0)
	{
		return(IR_SUCCESS);
	}
	else
	{
		CS_LOG_PRINTS(srcFile, "Error converting IR to snacc OS\n");
		return(IR_FAIL);
	}
}

IR_STATUS RescASN_IRToSnaccBS(CHAR *srcFile, AsnBits *bits, BIT_STRING *bs)
{
	if (bs->len == 0)
	{
		return(IR_SUCCESS);
	}
	if (ASN_BitsAdd(bits, bs->sPtr, (USHORT)bs->len) == 0)
	{
		return(IR_SUCCESS);
	}
	else
	{
		CS_LOG_PRINTS(srcFile, "Error converting IR to snacc BS\n");
		return(IR_FAIL);
	}
}

IR_STATUS RescASN_IRToSnaccOI(CHAR *srcFile, AsnOid *oidEntry, 
	     OBJECT_IDENTIFIER *oi)
{
	if (oi->len == 0)
	{
		return(IR_SUCCESS);
	}
CS_LOG_PRINTS(IR_FILE, "Converting dotted string to snacc oi \n");
RescUT_DebugVS(srcFile, oi);
CS_LOG_PRINTV(IR_FILE, "oi len is %d\n", oi->len);
	if (ASN_DottedStringToOid(oidEntry,  oi->sPtr, oi->len) == 0)
	{
		return(IR_SUCCESS);
	}
	else
	{
		CS_LOG_PRINTS(srcFile, "Error converting IR to snacc OI\n");
		return(IR_FAIL);
	}
}

IR_STATUS RescASN_SnaccToIRVS(CHAR *srcFile, VISIBLE_STRING *vs, AsnOcts *octs )
{
	CHAR	*buf;
	ULONG	count;


	ASN_OctsGet(&buf, &count, octs);

	if (vs->sPtr != NULL)
	{
		RescASN_DestroyVS(srcFile, vs);
	}

	if (count == 0)
	{
		vs->len = 0;
		vs->sPtr = (CHAR *)NULL;
		return(IR_SUCCESS);
	}

	if (RescASN_AddVS(srcFile, vs, buf, (SHORT)count) == 0)
	{
		return(IR_SUCCESS);
	}
	else
	{
		CS_LOG_PRINTS(srcFile, "Error converting snacc to IR VS\n");
		return(IR_FAIL);
	}
}

IR_STATUS RescASN_SnaccToIROS(CHAR *srcFile, OCTET_STRING *os, AsnOcts *octs )
{
	CHAR	*buf;
	ULONG	count;


	ASN_OctsGet(&buf, &count, octs);

	if (os->sPtr != NULL)
	{
		RescASN_DestroyOS(srcFile, os);
	}

	if (count == 0)
	{
		os->len = 0;
		os->sPtr = (CHAR *)NULL;
		return(IR_SUCCESS);
	}

	if (RescASN_AddOS(srcFile, os, buf, (SHORT)count) == 0)
	{
		return(IR_SUCCESS);
	}
	else
	{
		CS_LOG_PRINTS(srcFile, "Error converting snacc to IR OS\n");
		return(IR_FAIL);
	}
}

IR_STATUS RescASN_SnaccToIRBS(CHAR *srcFile, BIT_STRING *bs, AsnBits *bits)
{
	CHAR	*buf;
	USHORT	bitCount;
	USHORT	byteCount;
	USHORT	count;


	ASN_BitsGet(&buf, &byteCount, &bitCount, bits);

	count = byteCount * 8 + bitCount;
	
	if (bs->sPtr != NULL)
	{
		RescASN_DestroyBS(srcFile, bs);
	}

	if (count == 0)
	{
		bs->len = 0;
		bs->sPtr = (CHAR *)NULL;
		return(IR_SUCCESS);
	}
	if (RescASN_AddVS(srcFile, bs, buf, (SHORT)count) == 0)
	{
		return(IR_SUCCESS);
	}
	else
	{
		CS_LOG_PRINTS(srcFile, "Error converting snacc to IR OS\n");
		return(IR_FAIL);
	}
}

IR_STATUS RescASN_SnaccToIROI(CHAR *srcFile, OBJECT_IDENTIFIER *oi, 
  	     AsnOid *oid)
{
	CHAR	buf[100];


    	if (ASNOID_PRESENT(oid))
	{
		/* Convert the Snacc object identifier to a dotted string */
		ASN_AsnOidToDottedString(oid, buf);

		if (oi->sPtr != NULL)
		{
			RescASN_DestroyOI(srcFile, oi);
		}

		if (buf[0] == (CHAR)NULL)
		{
			oi->sPtr = (CHAR *)NULL;
			oi->len = 0;
		}
		else
		{
			/* Convert the dotted string to an IR object identifier. */
			RescASN_AddOI(srcFile, oi, buf, strlen(buf));
		}
	}
	return(IR_SUCCESS);
}

void *RescASN_CloneVS(CHAR *srcFile, void *data)
{
	VISIBLE_STRING *srcVs;
	VISIBLE_STRING	*destVs;

	srcVs = (VISIBLE_STRING *)data;

	destVs = RescASN_CreateVS(srcFile);

	if (destVs == NULL)
	{
		CS_LOG_PRINTS(srcFile, "Error cloning visible string\n");
		return(destVs);
	}

	RESC_ASN_MALLOCED(destVs);
	RESC_ASN_INIT_VS(destVs);
	RescASN_CopyVS(srcFile, destVs, srcVs);
	return(destVs);
}

#define MAX_STR_CMP_SIZE 	60
SHORT RescASN_CompareVS(CHAR *srcFile, VISIBLE_STRING *v1, VISIBLE_STRING *v2)
/*
 * This function returns the result of a strncmp.
 */
{

	CHAR	str1[MAX_STR_CMP_SIZE];
	CHAR	str2[MAX_STR_CMP_SIZE];


	VSToStrNullTerm(str1, v1, MAX_STR_CMP_SIZE);
	VSToStrNullTerm(str2, v2, MAX_STR_CMP_SIZE);
	
	return(strncmp(str1,str2,MAX_STR_CMP_SIZE));
}

IR_STATUS RescASN_CmpOIToStrOI(CHAR *srcFile, OBJECT_IDENTIFIER *oi, 
    CHAR *str)
{

	CHAR	str1[MAX_STR_CMP_SIZE];


	VSToStrNullTerm(str1, oi, MAX_STR_CMP_SIZE);
	
	if (strcmp(str1,str) == 0)
	{
		return(IR_SUCCESS);
	}
	else
	{
		return(IR_FAIL);
	}
}
