
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

/***************************************************************************
 * UNIT: RescUT_DestroyRPNSearchReq
 *
 * DESCRIPTION:
 *	This unit contains the functions need to destroy a search request
 *	that is passed from ua to op via IRO API
 *	
 *
 * CONTENTS:
 *	RescUT_DestroyRPNSearchReq
 *	
 *
 * NOTES:
 *
 * $Header$
 * $Log$
 *
/***************************************************************************/

#include <ir.h>
#include <string.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <op/ir_op.h>
#include <asn_tool.h>
#include <ir_oid.h>
#include <resclist.h>

static void RescUT_SrchDestroyRPNOperand(IR_RPN_ELEMENT *srcElem);
static void RescUT_SrchDestroyRPNStructSeq(IR_RPN_ELEMENT *srcElem);
static void RescUT_SrchDestroyRPNSearchQuery(IR_RPN_ELEMENT *srcQuery);

/****************************************************************************
 * FUNCTION:  RescUT_SrchDestroyRPNSearchQuery
 *
 * PURPOSE:
 *	This function is used to destroy an IR RPN query struct.
 *
 * INPUT:
 *	IR_RPN_ELEMENT	- src RPN search query 
 *
 * OUTPUT:
 *	RPNQuery*			- pointer to an apdu type 1 query
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/
void RescUT_SrchDestroyRPNSearchQuery(IR_RPN_ELEMENT *srcQuery)
{
	/* 
	 * Start with the first element and perform a recursive conversion
	 * of all elements.
	 */
	if (srcQuery->rpnElemType == IR_RPN_NODE_SIMPLE)
	{
		RescUT_SrchDestroyRPNOperand(srcQuery);
	}
	else
	{
		RescUT_SrchDestroyRPNStructSeq(srcQuery);
	}
}

/****************************************************************************
 * FUNCTION:  RescUT_SrchDestroyRPNOperand
 *
 * PURPOSE:
 *	This function is used to destroy an IR RPN query operand.
 *
 * INPUT:
 *	IR_RPN_ELEMENT	- src RPN search query element
 *
 * OUTPUT:
 *	NONE
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/
void RescUT_SrchDestroyRPNOperand(IR_RPN_ELEMENT *srcElem)
{

	if (srcElem->rpnOperand->simpleType == IR_RPN_SIMPLE_ATTR)
	{
		RescASN_DestroyOS(IR_FILE, 
		    &srcElem->rpnOperand->attrPlusTerm.term);

		RescList_Destroy(IR_FILE, 
		    &srcElem->rpnOperand->attrPlusTerm.attrList);
	}
	else
	{
		RescASN_DestroyVS(IR_FILE, &srcElem->rpnOperand->resultSetId);
	}

	if (RESC_ASN_IS_MALLOCED(srcElem))
	{
		RescIR_Free(IR_FILE, (CHAR *)srcElem);
	}

}

/****************************************************************************
 * FUNCTION:  RescUT_SrchDestroyRPNStructSeq
 *
 * PURPOSE:
 *	This function is used to destroy an IR RPN query structure.
 *
 * INPUT:
 *	IR_RPN_ELEMENT	- src RPN search query element
 *
 * OUTPUT:
 *	NONE
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/
void RescUT_SrchDestroyRPNStructSeq(IR_RPN_ELEMENT *srcElem)
{
	IR_RPN_COMPLEX_NODE 	*srcNode;
	IR_RPN_ELEMENT		*newSrc;

	srcNode = srcElem->rpnStructSeq;

	newSrc = (IR_RPN_ELEMENT *)srcNode->rpnElem1;

	RescUT_SrchDestroyRPNSearchQuery(newSrc);

	if (RESC_ASN_IS_MALLOCED(newSrc))
	{
		RescIR_Free(IR_FILE, (CHAR *)newSrc);
	}

	newSrc = (IR_RPN_ELEMENT *)srcNode->rpnElem2;

	RescUT_SrchDestroyRPNSearchQuery(newSrc);

	if (RESC_ASN_IS_MALLOCED(newSrc))
	{
		RescIR_Free(IR_FILE, (CHAR *)newSrc);
	}


}

/****************************************************************************
 * FUNCTION:  RescUT_DestroyIROSearchReq
 *
 * PURPOSE:
 *	This function is used to destroy a copy of IRO_SEARCH Request.
 *
 * INPUT:
 *	IRO_SEARCH_REQ	- src search request 
 *
 * OUTPUT:
 *	IRO_SEARCH_REQ	- dest search request 
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

SHORT RescUT_DestroyIROSearchReq(void *data)
{
	IRO_SEARCH_REQ	*req;

	req = (IRO_SEARCH_REQ *)data;

	RescASN_DestroyVS(IR_FILE, &req->resultSetName);

	RescList_Destroy(IR_FILE, &req->dbNameList);

	RescASN_DestroyVS(IR_FILE, &req->smallSetList.generic);
	RescList_Destroy(IR_FILE, &req->smallSetList.dbSpecific);

	RescASN_DestroyVS(IR_FILE, &req->mediumSetList.generic);
	RescList_Destroy(IR_FILE, &req->mediumSetList.dbSpecific);

	RescASN_DestroyOS(IR_FILE, &req->recordSyntax);

	RescASN_DestroyOI(IR_FILE, &req->query.attributeSetId);

	/* Destroy the rpn tree. */
	RescUT_SrchDestroyRPNSearchQuery(&req->query.rpnElem);

	return(0);
}
