/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Association Control timer control.
 *
 * DESCRIPTION:
 *	The Association Control Timer Control CSU 
 *	Handles timeouts for associations.
 *
 * CONTENTS:
 *	UaAc_ResetTimer
 *	UaAc_SetTimerDuration
 *	UaAc_GetTimerDuration
 *	UaAc_CheckTimeout
 *	UaAc_RemoveTimedItem
 *
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <resc.h>
#include <ua.h>
#include <ua/uaepmsg.h>


static SHORT TimerDuration = 600;

typedef struct {
	IR_ASSOC_ID	assocId;
	SHORT		click;
	BOOLEAN		used;
} UAAC_TIMER_STRUCT;

static UAAC_TIMER_STRUCT TimerList[20] = {
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE},
	{ 0, 0, FALSE}
};

void UaAc_ResetTimer(
    IR_ASSOC_ID assocId)   /* Association to update . */
{
	SHORT		counter;
	SHORT		firstFree;

	firstFree = -1;

	counter = 0;
	while ((counter < 20) && (TimerList[counter].assocId != assocId))
	{
		if ((firstFree == -1) && (TimerList[counter].used == FALSE))
		{
			firstFree = counter;
		}
		counter++;
	}

	if (counter < 20) 
	{
		TimerList[counter].click = 0;
	}
	else if (firstFree != -1)
	{
		TimerList[firstFree].assocId = assocId;
		TimerList[firstFree].used = TRUE;
		TimerList[firstFree].click = 0;
	}
	else
	{
		UaHec_UserMsg(IR_FILE, UAEP_MSG_TYPE_AC, 
		    MSG_ID_AC_TIMEOUT,
	    	    (CHAR *)NO_STRING, 
		    (USHORT)assocId);
		UaAc_AbortAssociation(assocId);
	}
}

void UaAc_SetTimerDuration(SHORT newDuration)
{
	TimerDuration = newDuration;
}

SHORT UaAc_GetTimerDuration(void)
{
	return TimerDuration;
}

void UaAc_CheckTimeout(void)
{
	SHORT count;

	if (TimerDuration < 0)
	{
		return;
	}
	for (count = 0; count < 20; count ++)
	{
		TimerList[count].click++;
		if ((TimerList[count].used) &&
		    (TimerList[count].click > TimerDuration))
		{
			TimerList[count].used = FALSE;
			UaHec_UserMsg(IR_FILE, UAEP_MSG_TYPE_AC, 
			    MSG_ID_AC_TIMEOUT,
		    	    (CHAR *)NO_STRING, 
			    (USHORT)TimerList[count].assocId);
			UaAc_AbortAssociation(TimerList[count].assocId);
			TimerList[count].assocId = 0;
		}
		
	}
}

void UaAc_RemoveTimedItem(IR_ASSOC_ID assocId)
{
	SHORT counter;

	counter = 0;
	while ((counter < 20) && (TimerList[counter].assocId != assocId))
	{
		counter++;
	}

	if ((counter< 20) && TimerList[counter].assocId == assocId)
	{
		TimerList[counter].used = FALSE;
		TimerList[counter].assocId = 0;
		TimerList[counter].click = 0;
	}
	
}
