/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Resource and Access Control status commands
 *
 * DESCRIPTION:
 *	The Resource and Access Control Resource Status CSU obtains 
 *  	association status information from the user and attempts to 
 *	perform the required status operations.
 *
 * CONTENTS:
 *	UaArc_ResourceStatus
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <string.h>
#include <ua.h>
#include <resc.h>
#include <ir_oid.h>

void static sendTriggerReq(UAEP_DIALOG *, IRO_TRIGGER_CNTL_OPER, BOOLEAN );

/****************************************************************************
 * FUNCTION: UaArc_ResourceStatus
 *
 * PURPOSE:
 *	The User Application Resource and Access Control Resuorce Status
 *	function obtains a target and resorce control operation to be 
 *	performmed and attempts to statify that operation.
 *
 * INPUT:
 *	There is no input to this function.
 *
 * OUTPUT:
 *	This function returns an IR_STATUS which returns a non-zero
 *	value on error. A call to UaHec_ErrorControl is required
 *	to inform the user of the problem.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

IR_STATUS UaArc_ResourceStatus( UAEP_DIALOG *arcInfo) 
/* from which menu the item was selected. */
{
	IRO_RSRC_RPRT_REQ rRReq;	/* Resource Report Request */
	UAEP_SYSTEM_INFO_STRUCT *taskWin; /* Session information */
	IRO_ASSOC_FULL_INFO_STRUCT infoStruct;

	CS_LOG_PRINTS(IR_FILE, "Executing status command\n");
	if (arcInfo->data.arcStatus.assocId == (IR_ASSOC_ID)IR_UNKNOWN_ID_TYPE)
	{
		return(0);
	}
	/* Get the related information for the select association */
	RESC_ASN_NO_MALLOC(&infoStruct);

	if (UaAc_GetAssociationInfoFull(arcInfo->data.arcStatus.assocId,
	    &infoStruct) == IR_FAIL)
	{
		arcInfo->data.arcStatus.assocId = IR_UNKNOWN_ID_TYPE;
		return(0);
	}


	switch(arcInfo->data.arcStatus.buttonSel)
	{
	case ARC_ABORT_ASSOC:
		UaAc_AbortAssociation(arcInfo->data.arcStatus.assocId);
		break;

	case ARC_RESOURCE_REPORT:

		if (infoStruct.assocStruct.state != IRO_ASSOC_STATE_IDLE)
		{
			/* Trigger a resource report */
			sendTriggerReq(arcInfo, IRO_TRIGGER_RSRC_RPRT,
			    FALSE);
		}
		else
		{
			/* Request a resource report */
			RESC_ASN_NO_MALLOC(&rRReq);
			RESC_ASN_NO_MALLOC(&rRReq.rsrcRprtFmt);
			rRReq.assocId = arcInfo->data.arcStatus.assocId;
			RescASN_AddOI(IR_FILE, &rRReq.rsrcRprtFmt,
		    	    OID_RESOURCE_REPORT_FMT_BIB1,
			    LENPLUSNULL(OID_RESOURCE_REPORT_FMT_BIB1));

			taskWin = 
			    (UAEP_SYSTEM_INFO_STRUCT *)get_app_data(TASK_WIN);
			if (IRO_ResourceReport(taskWin->userId, &rRReq) 
		    	    == IR_FAIL)
			{
				UaHec_ReportIROError(IR_FILE, 
				    IRO_RESOURCEREPORT_FUNC, 0);
			}

			UaAc_CommStatusUpdate();
			/* Free any malloced information */
			RescASN_DestroyOI(IR_FILE, &rRReq.rsrcRprtFmt);
		}
		break;

	case ARC_TRIG_RESOURCE_CTRL:
		/*
		 * User wants to trigger a resource control. 
	 	 */
		if (infoStruct.assocStruct.state != IRO_ASSOC_STATE_IDLE)
		{
			sendTriggerReq(arcInfo, IRO_TRIGGER_RSRC_CNTL,
			    FALSE);
		}
		break;
		
	case ARC_STOP_OPERATION:
		/*
 		 * Use wishes to use the trigger resource control request
		 * to cancel the current operation.
		 */
		if (infoStruct.assocStruct.state != IRO_ASSOC_STATE_IDLE)
		{
			sendTriggerReq(arcInfo, IRO_TRIGGER_CANCEL,
			    TRUE);
		}
		break;
	default:
		break;
	}

	UaAc_DestroyFullInfoStruct(&infoStruct);
	
	return (IR_SUCCESS);
}

void static sendTriggerReq(UAEP_DIALOG *arcInfo, IRO_TRIGGER_CNTL_OPER oper,
		BOOLEAN	resultSetWanted)
{
	IRO_TRIGGER_CNTRL_REQ trcReq;	/* Trigger Resource Control Request */
	UAEP_SYSTEM_INFO_STRUCT *taskWin; /* Session information */

	RESC_ASN_NO_MALLOC(&trcReq);
	RESC_ASN_NO_MALLOC(&trcReq.resourceReportFmt);

	RescASN_AddOI(IR_FILE, &trcReq.resourceReportFmt,
	    OID_RESOURCE_REPORT_FMT_BIB1,
	    LENPLUSNULL(OID_RESOURCE_REPORT_FMT_BIB1));

	trcReq.assocId = arcInfo->data.arcStatus.assocId;
	trcReq.operation = oper;
	trcReq.resultSetWanted = resultSetWanted;

	taskWin = (UAEP_SYSTEM_INFO_STRUCT *)get_app_data(TASK_WIN);
	if (IRO_TriggerControl(taskWin->userId, &trcReq) == IR_FAIL)
	{
		UaHec_ReportIROError(IR_FILE, IRO_TRIGGERCONTROL_FUNC, 
	    	    0);
	}

	RescASN_DestroyOI(IR_FILE, &trcReq.resourceReportFmt);
}
