/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Help and Error Control Error Reporting
 *
 * DESCRIPTION:
 *	This unit is used for error reporting based on both UA and IRO 
 *	detected errors.
 *
 * CONTENTS:
 *	UaHec_ReportIROError
 *	UaHec_ReportUAError
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <ua.h>

#ifndef XVT_MSDOS
#include <stdio.h>
extern FILE 		*CS_LogFp;
#endif

typedef struct
{
	SHORT	funcType;
	SHORT	recType;
	CHAR	*funcName;
	CHAR	*recName;
} IRO_ERR_TABLE;

static IRO_ERR_TABLE errTbl[]=
{

	{IRO_INITIALIZE_FUNC,-1, "IRO_Initialize", ""},
	{IRO_COMM_EVENT_FUNC,-1, "IRO_CommEvent", ""},
	{IRO_ASSOCIATION_FUNC,-1, "IRO_Association", ""},
	{IRO_ABORT_FUNC,-1, "IRO_Abort", ""},
	{IRO_TERMINATE_FUNC,-1, "IRO_Terminate", ""},
	{IRO_SEARCH_FUNC,-1, "IRO_Search", ""},
	{IRO_PRESENT_FUNC,-1, "IRO_Present", ""},
	{IRO_DELETE_FUNC,-1, "IRO_Delete", ""},
	{IRO_WORKSPACEINFO_FUNC,-1, "IRO_WorkSpaceInfo", ""},
	{IRO_ACCESSCONTROL_FUNC,-1, "IRO_AccessControl", ""},
	{IRO_RESOURCECONTROL_FUNC,-1, "IRO_ResourceControl", ""},
	{IRO_TRIGGERCONTROL_FUNC,-1, "IRO_TriggerControl", ""},
	{IRO_RESOURCEREPORT_FUNC,-1, "IRO_ResourceReport", ""},
	{IRO_USERINFO_FUNC,-1, "IRO_UserInfo", ""},
	{IRO_TARGETINFO_FUNC,-1, "IRO_TargetInfo", ""},
	{IRO_DATABASEINFO_FUNC,-1, "IRO_DatabaseInfo", ""},
	{-1, -1, "Unknown IRO_function", "unknown request"}
};



/****************************************************************************
 * FUNCTION: UaAc_ReportIROError
 *
 * PURPOSE:
 *	This function is used to report IRO errors to the user.
 *
 * INPUT:
 *	UAHEC_IRO_FUNC		- iro function called
 *	UAHEC_IRO_REC_TYPE	- function request type
 *	none - for now.
 *
 * OUTPUT:
 *	none.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

void UaHec_ReportIROError(CHAR *src, IRO_FUNC_NAME func,
    IRO_FUNC_REC_TYPE recType)
{
	extern BOOLEAN	UaHec_DispErr();
	SHORT	i;
	IRO_ERROR_REPLY	err;


	if (!UaHec_DispErr())
	{
		return;
	}
	IRO_ErrorReport(&err);

	i = 0;
	while ((errTbl[i].funcType != (-1)) &&
	    (errTbl[i].funcType != func))
	{
		i++;
	}
	if (errTbl[i].funcType == (-1))
	{
#ifdef XVT_MSDOS
		xvt_note("%s, IRO Error: %s, %s, Level %d, Code %d, %s", 
		    src,
		    errTbl[i].funcName, errTbl[i].recName,
		    err.errorLevel, err.errorCode, err.errorString);
#else
		fprintf(CS_LogFp, "%s, IRO Error: %s, %s, Level %d, Code %d, %s", 
		    src,
		    errTbl[i].funcName, errTbl[i].recName,
		    err.errorLevel, err.errorCode, err.errorString);
#endif
	}
	else
	{
#ifdef XVT_MSDOS
		xvt_note("%s, IRO Error: %s, recType %d, Level %d, Code %d, %s", 
		    src,
		    errTbl[i].funcName, recType,
		    err.errorLevel, err.errorCode, err.errorString);
#else
		fprintf(CS_LogFp, "%s, IRO Error: %s, recType %d, Level %d, Code %d, %s", 
		    src,
		    errTbl[i].funcName, recType,
		    err.errorLevel, err.errorCode, err.errorString);
#endif
	}

}

/****************************************************************************
 * FUNCTION: UaAc_ReportUAError
 *
 * PURPOSE:
 *	This function is used to report UA detected errors to the user.
 *
 * INPUT:
 *	CHAR *			- source string IR_FILE
 *	CHAR *			- msg string
 *
 * OUTPUT:
 *	none.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

void UaHec_ReportUAError(CHAR *src, CHAR *string)
{
	CHAR *s;
	CHAR *t;

	if (src != (CHAR *)NULL) 
	{
		s = src;
	}
	else
	{
		s = "";
	}

	if (string != (CHAR *)NULL)
	{
		t = string;
	}
	else
	{
		t = "";
	}

#ifdef XVT_MSDOS
	xvt_note("%s, %s", s, t);
#else
	fprintf(CS_LogFp, "%s, %s", s, t);
#endif
}
