
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Record Display Fetch Dialog control
 *
 * DESCRIPTION:
 *
 * CONTENTS:
 *	UaRdc_FrDlgCreate
 *	UaRdc_FrDlgDestroy
 *	UaRdc_FrDlgGetSelect
 *	UaRdc_FrDlgFetch
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
 *
/***************************************************************************/

#include <ir.h>
#include <uaep.h>
#include <ir_api.h>
#include <ua.h>
#include <ir_oid.h>
#include <resc.h>
#include <debug.h>
#include <resclist.h>

static UASC_DB_INFO_STRUCT	*UaRdc_TgInfo;

static SLIST	recSyntaxList=(SLIST)NULL;
static SLIST	recCompList=(SLIST)NULL;

/****************************************************************************
 * FUNCTION: UaRdc_FrDlgCreate
 *
 * PURPOSE:
 *
 * INPUT:
 *
 * OUTPUT:
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

IR_STATUS UaRdc_FrDlgCreate(WINDOW win, UAEP_DIALOG *info)
{
	IR_LIST			assocList;
	IRO_ASSOC_STRUCT	*assocElem;
	CHAR			tmpBuf[IR_LEN_100];
	UASC_DB_INFO_STRUCT	*tgDataElem;
	WINDOW			ctlWin;
	IRO_WS_HEADER_INFO	*wsInfo;
	CHAR			listData[IR_LEN_100];
	LONG			presentNum;

	UaRdc_TgInfo = (UASC_DB_INFO_STRUCT *)NULL;

	if (!recSyntaxList)
	{
		recSyntaxList = slist_new();
	}
	if (!recCompList)
	{
		recCompList = slist_new();
	}

	/* Get the association information for the corresponding assocId */
	RESC_ASN_NO_MALLOC(&assocList);
	RESC_LIST_INIT(&assocList);
	if (UaAc_GetAssociationInfo(UAAC_GET_ONE_ASSOC, 
		info->data.display.assocId, &assocList) == IR_FAIL)
	{
		xvt_note("Could not get assoc information");
		CS_LOG_PRINTV(IR_FILE, 
		    "Couldn't get assoc info for assocId %d\n",
		    info->data.display.assocId);
		return(IR_FAIL);
	}

	if (RESC_LIST_SIZE(&assocList) != 1)
	{
		xvt_note("No targets currently connected");
		RescList_Destroy(IR_FILE, &assocList);
		return(IR_FAIL);
	}

	/* 
	 * Get the target information for the assoc item just received.
	 */

	RESC_LIST_SET_CUR(&assocList, (SHORT)0);
	
	assocElem = RESC_LIST_GET(&assocList);

	/* Fill the assoc info into the target data struct */
	tgDataElem = UaSc_DbCreateDbInfoStruct(IR_FILE);
	if (tgDataElem == NULL)
	{
		RescList_Destroy(IR_FILE, &assocList);
		return(IR_FAIL);
	}

		
	UaSc_DbInitDbInfoStruct(tgDataElem);
	tgDataElem->assocId = assocElem->assocId;
	UaSc_DbFillTargetInfo(assocElem->targetId, tgDataElem);
	UaRdc_TgInfo = tgDataElem;

	/* Done with the assocList */
	RescList_Destroy(IR_FILE, &assocList);

	/* Now get the record syntax supported by this target. */
	UaSc_DispSyntaxList(&tgDataElem->tgInfo.dbInfo, recSyntaxList);
	UaSc_DispElemSetList(&tgDataElem->tgInfo.dbInfo, recCompList);

	ctlWin = get_ctl_window(win, FR_DLG_SYNTAX_LB);
	win_list_clear(ctlWin);
	win_list_add(ctlWin, -1, (STR)recSyntaxList);
	win_list_set_sel(ctlWin, 0, TRUE);

	ctlWin = get_ctl_window(win, FR_DLG_REC_COMP_LB);
	win_list_clear(ctlWin);
	win_list_add(ctlWin, -1, (STR)recCompList);
	win_list_set_sel(ctlWin, 0, TRUE);

	/* Display the work space information */
	wsInfo = info->data.display.wsHdrInfo;

	ctlWin = get_ctl_window(win, FR_DLG_LOCAL_NAME_EDIT);
	VSToStrNullTerm(tmpBuf, &wsInfo->name, IR_LEN_50);
	sprintf(listData, "SEARCH_%1d,%s",  wsInfo->wsId, tmpBuf);
	set_title(ctlWin, listData);

	ctlWin = get_ctl_window(win, FR_DLG_LOCAL_COUNT_EDIT);
	sprintf(listData, "%1ld",  wsInfo->numWSRecords);
	set_title(ctlWin, listData);

	ctlWin = get_ctl_window(win, FR_DLG_REMOTE_NAME_EDIT);
	VSToStrNullTerm(listData, &wsInfo->name, IR_LEN_50);
	set_title(ctlWin, listData);

	
	ctlWin = get_ctl_window(win, FR_DLG_REMOTE_COUNT_EDIT);
	sprintf(listData, "%1ld",  wsInfo->totalTargetRecords);
	set_title(ctlWin, listData);


	ctlWin = get_ctl_window(win, FR_DLG_REMOTE_NEXT_EDIT);
	sprintf(listData, "%1ld",  wsInfo->nextTargetRec);
	set_title(ctlWin, listData);

	ctlWin = get_ctl_window(win, FR_DLG_REC_COUNT_EDIT);
	if (tgDataElem->userTgInfo.defPresentNumber >
	    (LONG)wsInfo->totalTargetRecords - (LONG)wsInfo->nextTargetRec)
	{
		presentNum = 
		    wsInfo->totalTargetRecords - wsInfo->nextTargetRec;
		if (presentNum != 0)
		{
			presentNum++;
		}
	}
	else
	{
		presentNum = tgDataElem->userTgInfo.defPresentNumber;
	}
	if (presentNum < 0)
	{
		presentNum = 0;
	}
	sprintf(listData, "%1ld",  presentNum);
	set_title(ctlWin, listData);

	ctlWin = get_ctl_window(win, FR_DLG_START_REC_EDIT);
	sprintf(listData, "%1ld",  wsInfo->nextTargetRec);
	set_title(ctlWin, listData);

	set_front_window(get_ctl_window(win, FR_DLG_SYNTAX_LB));

	return(IR_SUCCESS);
}


IR_STATUS UaRdc_FrDlgDestroy(WINDOW win)
{
	if (recSyntaxList)
	{
		slist_dispose(recSyntaxList);
		recSyntaxList = (SLIST)NULL;
	}
	if (!recCompList)
	{
		slist_dispose(recCompList);
		recCompList = (SLIST)NULL;
	}

	UaSc_DbDestroyDbInfoStruct(IR_FILE, UaRdc_TgInfo);
	UaRdc_TgInfo = (UASC_DB_INFO_STRUCT *)NULL;

	return(IR_SUCCESS);
}

IR_STATUS UaRdc_FrDlgFetch(WINDOW win, UAEP_DIALOG *info)
{
	IRO_PRESENT_REQ	pReq;
	WINDOW			ctlWin;
	VISIBLE_STRING		*vs;
	CHAR			tmpBuf[IR_LEN_100];
	
	
	RESC_ASN_NO_MALLOC(&pReq);
	RESC_ASN_NO_MALLOC(&pReq.resultSetName);
	RESC_ASN_INIT_VS(&pReq.resultSetName);
	RESC_ASN_NO_MALLOC(&pReq.recordSyntax);
	RESC_ASN_INIT_OI(&pReq.recordSyntax);

	pReq.elemSetNameType = IR_ELEM_SET_NAME_NONE;
	RESC_ASN_NO_MALLOC(&pReq.elemSetList.generic);
	RESC_ASN_INIT_VS(&pReq.elemSetList.generic);
	RESC_ASN_NO_MALLOC(&pReq.elemSetList.dbSpecific);
	RESC_LIST_INIT(&pReq.elemSetList.dbSpecific);

	ctlWin = get_ctl_window(win, FR_DLG_SYNTAX_LB);
	UaSc_GetSyntaxFromList(ctlWin, recSyntaxList, 
	    &pReq.recordSyntax);
	if (pReq.recordSyntax.len == 0)
	{
		xvt_note("Must select a syntax");
		return(IR_FAIL);
	}


	if (UaRdc_TgInfo->tgInfo.elementSetNamingSupported == TRUE)
	{
		vs = RescASN_CreateVS(IR_FILE);
		ctlWin = get_ctl_window(win, FR_DLG_REC_COMP_LB);
		UaSc_GetElemSetNameFromList(ctlWin, recCompList, vs);

		if (vs->len == 0)
		{
			xvt_note("Must select a composition type");
			RescASN_DestroyOI(IR_FILE, &pReq.recordSyntax);
			return(IR_FAIL);
		}
		else
		{
			pReq.elemSetNameType = IR_ELEM_SET_NAME_GENERIC;
			RescASN_CopyVS(IR_FILE, &pReq.elemSetList.generic, vs);
		}
	}

	CS_LOG_PRINTS(IR_FILE, "Creating result set name\n ");
	

	/* Use result set name from work space */
	RescASN_CopyVS(IR_FILE, &pReq.resultSetName,
    	    &info->data.display.wsHdrInfo->name);

	pReq.assocId = info->data.display.assocId;
	pReq.wsId = info->data.display.wsId;

	CS_LOG_PRINTS(IR_FILE, "Created result set name\n ");

	ctlWin  = get_ctl_window(win, FR_DLG_START_REC_EDIT);
	get_title(ctlWin, tmpBuf, 20);
	CS_LOG_PRINTV(IR_FILE, "Got start record %s\n", tmpBuf);
	pReq.startRec = atol(tmpBuf);

	ctlWin  = get_ctl_window(win, FR_DLG_REC_COUNT_EDIT);
	get_title(ctlWin, tmpBuf, 20);
	CS_LOG_PRINTV(IR_FILE, "Got record count %s\n", tmpBuf);
	pReq.numRecords = atol(tmpBuf);

	CS_LOG_PRINTS(IR_FILE, "Sending present request \n ");
	
	if (IRO_Present(UaEp_InfoUserId(), &pReq) == IR_FAIL)
	{
		UaHec_ReportIROError(IR_FILE, IRO_PRESENT_FUNC, 0);
		return(IR_FAIL);
	}
	UaAc_CommStatusUpdate();
	return(IR_SUCCESS);
}
