
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Record Display Result
 *
 * DESCRIPTION:
 *	The Record Display Result CSU obtains result set information
 *	from the user and attempts to display more records from
 *	the result set.
 *
 * CONTENTS:
 *	UaRdc_DisplayResult
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
 *
/***************************************************************************/

#include <ir.h>
#include <uaep.h>
#include <ua.h>
#include <resc.h>
#include <debug.h>

/****************************************************************************
 * FUNCTION: UaRdc_DisplayResult
 *
 * PURPOSE:
 *	The User Application Record Display Result
 *	function obtains a target and result set from the user and
 *	attempts to Display more records from the result set.
 *
 * INPUT:
 *	There is no input to this function.
 *
 * OUTPUT:
 *	This function returns an IR_STATUS which returns a non-zero
 *	value on error. A call to UaHec_ErrorControl is required
 *	to inform the user of the problem.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

IR_STATUS UaRdc_DisplayResult(void)
{
	IR_STATUS	returnCode;	/* Return value for function.  */
	UAEP_DIALOG	display;	/* Display info gathered from user. */

	returnCode = IR_SUCCESS;

	memset(&display, (CHAR)NULL, sizeof(UAEP_DIALOG));

	create_res_dialog(WD_MODAL, DLG_RESULT_DISPLAY, 
	    EM_ALL, dlg_110_eh, (long)&display);

	if (display.cancelled == TRUE)
	{
		return (returnCode);
	}
	CS_LOG_PRINTV(IR_FILE, "About to update display for wsId %d\n",
	    display.data.display.wsId);

	if (display.data.display.wsId == IR_UNKNOWN_ID_TYPE)
	{
		/* Cannot create work space. No corresponding op workspace */
		xvt_note("Trying to display an unselected workspace\n");
		returnCode = IR_FAIL;
	}
	else
	{
		/* Pop up the search screen */
		UaEp_UpdateWin(display.data.display.wsId);
		returnCode = IR_SUCCESS;
	}

	return (returnCode);
}
