/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: Search Get Database
 *
 * DESCRIPTION:
 *
 *	The Search Get Database CSU obtains the name of the database
 *	the user wants to apply to the current search. Will also 
 *	attempt to send the search at the users request.
 *
 * CONTENTS:
 *	UaSc_GetDatabase
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
 ***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <ua.h>
#include <resc.h>
#include <resclist.h>
#include <debug.h>
#include <uaep.h>


static IR_LIST	targetData;	/* of UASC_DB_INFO_STRUCT */
static SHORT   targetIndex;

IR_STATUS UaSc_DbInitDlg(WINDOW win, SLIST targetList, SLIST **dbLists)
{
	IR_LIST			assocList;
	SHORT			numAssocs;
	IRO_ASSOC_STRUCT	*assocElem;
	CHAR			tmpBuf[IR_LEN_100];
	UASC_DB_INFO_STRUCT	*tgDataElem;
	SHORT			tgIndex;
	SHORT			dbIndex;
	IR_DATABASE_INFO	*dbDataElem;
	SLIST			tmpList;
	SLIST			*dispDbList;

	RESC_ASN_NO_MALLOC(&targetData);
	RESC_LIST_INIT(&targetData);
	targetIndex = 0;

	RESC_ASN_NO_MALLOC(&assocList);
	RESC_LIST_INIT(&assocList);
	if (UaAc_GetAssociationInfo(UAAC_GET_ALL_ASSOCS, IR_UNKNOWN_ID_TYPE,
	    &assocList) == IR_FAIL)
	{
		CS_LOG_PRINTS(IR_FILE, "Couldn't get assoc info for all targets\n");
		return(IR_FAIL);
	}

	if (RESC_LIST_SIZE(&assocList) == 0)
	{
		xvt_note("No targets currently connected");
		RescList_Destroy(IR_FILE, &assocList);
		return(IR_FAIL);
	}

	/* malloc an SLIST for each target's database list */
	numAssocs = RESC_LIST_SIZE(&assocList);
	dispDbList = (SLIST *)RescIR_Zmalloc(IR_FILE,
	     (sizeof(SLIST *) * numAssocs));
	*dbLists = dispDbList;
	     
	/* 
	 * For each target generate an slist containing the database
	 * name and a pointer to the database name structure.
	 */

	RESC_LIST_SET_FIRST(&assocList);
	for (tgIndex = 0; tgIndex < numAssocs; tgIndex++)
	{
		tmpList = slist_new();

		dispDbList[tgIndex] = tmpList;

		if (tmpList == NULL)
		{
			RescList_Destroy(IR_FILE, &assocList);
			return(IR_FAIL);
		}
		assocElem = RESC_LIST_GET(&assocList);

		/* Fill the assoc info into the target data struct */
		tgDataElem = UaSc_DbCreateDbInfoStruct(IR_FILE);
		if (tgDataElem == NULL)
		{
			RescList_Destroy(IR_FILE, &assocList);
			return(IR_FAIL);
		}

		
		UaSc_DbInitDbInfoStruct(tgDataElem);
		tgDataElem->assocId = assocElem->assocId;
		UaSc_DbFillTargetInfo(assocElem->targetId,
		    tgDataElem);

		/* Add the target information to the target list */
		VSToStrNullTerm(tmpBuf, &assocElem->name, 40);

		slist_add_sorted(targetList, tmpBuf,
		     (long)tgIndex, FALSE, FALSE);

		/* Add the database information to the database list */
		RESC_LIST_SET_FIRST(&tgDataElem->tgInfo.dbInfo);
		for (dbIndex = 0;
		    dbIndex < RESC_LIST_SIZE(&tgDataElem->tgInfo.dbInfo);
		    dbIndex++)
		{
			/* Get each database */
			dbDataElem = 
			    RESC_LIST_GET(&tgDataElem->tgInfo.dbInfo);

			VSToStrNullTerm(tmpBuf, &dbDataElem->databaseName, 40);

			slist_add_sorted(tmpList, tmpBuf,
			    (long)dbIndex, FALSE, FALSE);
			RESC_LIST_SET_NEXT(&tgDataElem->tgInfo.dbInfo);
		}

		/* Append target info to db_info_struct list */
		RescList_Append(IR_FILE, &targetData, tgDataElem,
		    UaSc_DbDestroyDbInfoStruct, NULL);

		/* Get next assoc/target */
		RESC_LIST_SET_NEXT(&assocList);
	}

	/* Destroy the assoc list info */
	RescList_Destroy(IR_FILE, &assocList);

	/* Display the target list to the user */
	win_list_clear(get_ctl_window(win, SD_TARGET_LB));
	win_list_add(get_ctl_window(win, SD_TARGET_LB),
	    -1, (char *)targetList);

	/* Select the first target to the user */
	win_list_set_sel(get_ctl_window(win, SD_TARGET_LB), 0, TRUE);

	/* Handle the target selection. (ie. display database list) */
	UaSc_DbHandleTargetList(win, targetList, *dbLists);

	if (numAssocs > 1)
	{
		/* Make the target list active */
		set_front_window(get_ctl_window(win, SD_TARGET_LB));
	}
	else
	{
		/* if only one association make the db list active */
		set_front_window(get_ctl_window(win, SD_DATABASE_LIST));
	}
	/* Select the first database to the user */
	win_list_set_sel(get_ctl_window(win, SD_DATABASE_LIST), 0, TRUE);

CS_LOG_PRINTS(IR_FILE, "Done creating database dialog\n");
	return(IR_SUCCESS);
}


void UaSc_DbDestroyDlg(WINDOW win, SLIST *dbList)
{
	SHORT index;

	for (index = 0; index < RESC_LIST_SIZE(&targetData); index++)
	{
		/* Destroy each target's database list */
		slist_dispose(dbList[index]);
	}
	
	/* Free the list of database SLISTs */
	RescIR_Free(IR_FILE, (CHAR *)dbList);

	/* Destroy the target data list */
	RescList_Destroy(IR_FILE, &targetData);

	/* Reintialize the target data list for next time */
	RESC_LIST_INIT(&targetData);
}

/*
 * This function will detect the selected databases from the database
 * list for the selected target.
 */
void UaSc_DbGetDatabases(WINDOW win, UAEP_DIALOG *info, SLIST tgList,
    SLIST *dbLists)
{
	SHORT	numSelect;
	SHORT	index;
	SHORT	numInList;
	VISIBLE_STRING *vs;
	IR_DATABASE_INFO *dbInfo;
	UASC_DB_INFO_STRUCT	*tg;
	WINDOW		listWin;
	long		data;
	
	/*
 	 * Find the target index for the currently selected target.
	 */
	listWin = get_ctl_window(win, SD_TARGET_LB);
	if (EpDialog_GetSelListData(listWin, tgList, &data) != IR_SUCCESS)
	{
		CS_LOG_PRINTS(IR_FILE, "No target selected\n");
		return;
	}
	else
	{
		targetIndex = (SHORT)data;
	}

	
	/* Get the corresponding target db info struct */
	RESC_LIST_SET_CUR(&targetData, targetIndex);
	tg = RESC_LIST_GET(&targetData);

	/*
	 * Get the number of databases selected for the target.
	 */
	listWin = get_ctl_window(win, SD_DATABASE_LIST);
	numSelect = win_list_count_sel(listWin);
	numInList = win_list_count_all(listWin);

	/* Destroy any previously selected databases */
	RescList_Destroy(IR_FILE, &info->data.database.dbNameList);

	/* Prepare to add newly selected databases */
	RESC_LIST_INIT(&info->data.database.dbNameList);

	/* Scan database list for selected databases. */
	index = 0;
	while ((numSelect > 0) && (index < numInList))
	{
		if (win_list_is_sel(listWin, index))
		{
			numSelect--;
			/* Get the selected database */
			slist_elt(dbLists[targetIndex], index, &data);
			RESC_LIST_SET_CUR(&tg->tgInfo.dbInfo, (SHORT)data);
			dbInfo = RESC_LIST_GET(&tg->tgInfo.dbInfo);
			vs = RescASN_CreateVS(IR_FILE);
			RescASN_CopyVS(IR_FILE, vs, 
			    &dbInfo->databaseName);
			RescList_Append(IR_FILE, 
			    &info->data.database.dbNameList,
			    vs, RescASN_DestroyVS, (void *)RescASN_CloneVS);
		}
		index++;
	}
	
CS_LOG_PRINTS(IR_FILE, "Done UaSc_DbGetDatases\n");
			
}
			

void UaSc_DbOptionsButton(WINDOW win, UAEP_DIALOG *info)
{

	UASC_DB_INFO_STRUCT	*tg;
	

	if (RESC_LIST_SIZE(&targetData) == 0)
	{
		xvt_beep();
	}

	RESC_LIST_SET_CUR(&targetData, targetIndex);

	tg = RESC_LIST_GET(&targetData);

	/*
	 * Get Options Info from User. Use same info structure.
	 */
	create_res_dialog(WD_MODAL, DLG_SEARCH_OPTIONS, 
	    EM_ALL, dlg_108_eh, (long)info);

	info->cancelled = FALSE;
}


/*
 * The following code is used to support the database options
 * dialog box. 
 */

static SLIST DoDlg_SmallSetList=(SLIST)NULL;
static SLIST DoDlg_MediumSetList=(SLIST)NULL;
static SLIST DoDlg_RecSyntaxList=(SLIST)NULL;

void UaSc_DbHandleTargetList(WINDOW win, SLIST tgList, SLIST *dbLists)
{
	WINDOW	tWin;
	WINDOW	dWin;

	long 	selData;
	SHORT	index;
	UASC_DB_INFO_STRUCT *tg;
	UAEP_DIALOG	*info;

	info = (UAEP_DIALOG *)get_app_data(win);

	tWin = get_ctl_window(win, SD_TARGET_LB);
	dWin = get_ctl_window(win, SD_DATABASE_LIST);

	if (EpDialog_GetSelListData(tWin, tgList, &selData) != IR_SUCCESS)
	{
		info->data.database.assocId = IR_UNKNOWN_ID_TYPE;
		return;
	}

	index = (SHORT)selData;

	/* Set global target selection identifier. */
	targetIndex = index;

	/* Get the selected target's db info struct */
	RESC_LIST_SET_CUR(&targetData, targetIndex);
	tg = RESC_LIST_GET(&targetData);
	
	info->data.database.assocId = tg->assocId;

	/* 
	 * Change the databases displayed to match the target.
	 */
	win_list_suspend(dWin);
	win_list_clear(dWin);
	win_list_add(dWin, -1, (STR)dbLists[index]);
	win_list_resume(dWin);

	if (tg->userTgInfo.defResultSetNamingSupported == TRUE)
	{
		info->data.database.replaceIndicator = TRUE;
	}
	else
	{
		info->data.database.replaceIndicator = FALSE;
	}
	info->data.database.smallSetUpperBound = 
	    tg->userTgInfo.defSmallSetUpperBound;
	info->data.database.largeSetPresentNum = 
	    tg->userTgInfo.defLargeSetLowerBound;
	info->data.database.mediumSetPresentNum = 
	    tg->userTgInfo.defPresentNumber;

	RescASN_DestroyOI(IR_FILE, &info->data.database.recordSyntax);
	RESC_ASN_INIT_OI(&info->data.database.recordSyntax);
	RescASN_CopyOI(IR_FILE, &info->data.database.recordSyntax,
	    &tg->userTgInfo.defPreferedRecSyntax);
	RescUT_DebugVS(IR_FILE, &info->data.database.recordSyntax);

	RescASN_DestroyVS(IR_FILE, &info->data.database.resultSetName);
	RESC_ASN_INIT_VS(&info->data.database.resultSetName);
	RescASN_AddVS(IR_FILE, &info->data.database.resultSetName,
	    "default", strlen("default"));
	CS_LOG_PRINTS(IR_FILE, "Added result set name\n");
	RescUT_DebugVS(IR_FILE, &info->data.database.resultSetName);
}


void UaSc_DoDlgInit(WINDOW win)
{
	UAEP_DIALOG	*info;
	UASC_DB_INFO_STRUCT	*tg;
	WINDOW		ctlWin;
	EP_DLG_DATABASE_SEARCH_STRUCT *defInfo;
	CHAR		tmpBuf[IR_LEN_50];

	info = (UAEP_DIALOG *)get_app_data(win);
	defInfo = &info->data.database;

	/* Create the lists */
	if (!DoDlg_SmallSetList)
	{
		DoDlg_SmallSetList = slist_new();
	}
	if (!DoDlg_MediumSetList)
	{
		DoDlg_MediumSetList = slist_new();
	}
	if (!DoDlg_RecSyntaxList)
	{
		DoDlg_RecSyntaxList = slist_new();
	}

	/* Get the current target from the target list */
	RESC_LIST_SET_CUR(&targetData, targetIndex);
	tg = RESC_LIST_GET(&targetData);


	/* Create the record syntax list */
	UaSc_DispSyntaxList(&tg->tgInfo.dbInfo, DoDlg_RecSyntaxList);

	ctlWin = get_ctl_window(win, SO_RECORD_SYNTAX_LIST);
	win_list_clear(ctlWin);
	win_list_add(ctlWin, -1, (STR)DoDlg_RecSyntaxList);
	win_list_set_sel(ctlWin, 0, TRUE);

	UaSc_DispElemSetList(&tg->tgInfo.dbInfo, DoDlg_SmallSetList);
	UaSc_DispElemSetList(&tg->tgInfo.dbInfo, DoDlg_MediumSetList);

	ctlWin = get_ctl_window(win, SO_SML_SET_ELM_LIST);
	win_list_clear(ctlWin);
	win_list_add(ctlWin, -1, (STR)DoDlg_SmallSetList);
	win_list_set_sel(ctlWin, 0, TRUE);
	
	ctlWin = get_ctl_window(win, SO_MED_SET_ELM_LIST);
	win_list_clear(ctlWin);
	win_list_add(ctlWin, -1, (STR)DoDlg_MediumSetList);
	win_list_set_sel(ctlWin, 0, TRUE);
	

	ctlWin = get_ctl_window(win, SO_RSLTSET_NAME_EDIT);
	VSToStrNullTerm(tmpBuf, &defInfo->resultSetName, 50);
	set_title(ctlWin, tmpBuf);

	
	ctlWin = get_ctl_window(win, SO_REPLACE_CHECKBX);
	win_check_box(ctlWin, defInfo->replaceIndicator);

	ctlWin = get_ctl_window(win, SO_LRG_SET_LOWER_BND);
	sprintf(tmpBuf, "%ld", defInfo->largeSetPresentNum);
	set_title(ctlWin, tmpBuf);

	ctlWin = get_ctl_window(win, SO_SML_SET_UPPER_BND);
	sprintf(tmpBuf, "%ld", defInfo->smallSetUpperBound);
	set_title(ctlWin, tmpBuf);

	ctlWin = get_ctl_window(win, SO_MED_SET_PRESENT);
	sprintf(tmpBuf, "%ld", defInfo->mediumSetPresentNum);
	set_title(ctlWin, tmpBuf);

	set_front_window(get_ctl_window(win, SO_RECORD_SYNTAX_LIST));
}

void UaSc_DoDlgDestroy(WINDOW win)
{
	slist_dispose(DoDlg_SmallSetList);
	slist_dispose(DoDlg_MediumSetList);
	slist_dispose(DoDlg_RecSyntaxList);

	
	DoDlg_SmallSetList = (SLIST)NULL;
	DoDlg_MediumSetList = (SLIST)NULL;
	DoDlg_RecSyntaxList = (SLIST)NULL;
}

void UaSc_DoDlgOK(WINDOW win, UAEP_DIALOG *info)
{
	UASC_DB_INFO_STRUCT	*tg;
	WINDOW		ctlWin;
	EP_DLG_DATABASE_SEARCH_STRUCT *defInfo;
	CHAR		tmpBuf[IR_LEN_50];

	defInfo = &info->data.database;

	ctlWin = get_ctl_window(win, SO_RSLTSET_NAME_EDIT);
	get_title(ctlWin, tmpBuf, IR_LEN_50);
	RescASN_DestroyVS(IR_FILE, &defInfo->resultSetName);
	RescASN_AddVS(IR_FILE, &defInfo->resultSetName, tmpBuf,
	    strlen(tmpBuf));
	
	ctlWin = get_ctl_window(win, SO_REPLACE_CHECKBX);
	defInfo->replaceIndicator = get_checked_state(ctlWin);
}
