/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

#include <ir.h>
#include <stdio.h>
#include <string.h>
#include <resc.h>
#include <xvt.h>
#include <ua\rpnquery.h>
#include <resclist.h>
#include <ir_oid.h>


#define new_struct(x)   (x *)RescIR_Zmalloc(IR_FILE, sizeof(x))

#ifdef XVTWS
#define strdup(x)               RescIR_StrDup(IR_FILE, x)
#endif

typedef struct {
    int     value;
    char    *name;
} TOKEN_TABLE;
#define table_size(x)   (sizeof(x)/sizeof(x[0]))

static TOKEN_TABLE  RPN_AttribComplete_Tokens[] = {
    RPN_AT_CPL_UNSPECIFIED        , NULL,
    RPN_AT_CPL_INCOMPLETE_SUBFIELD, "IncompleteSubfield",
    RPN_AT_CPL_COMPLETE_SUBFIELD  , "CompleteSubfield",
    RPN_AT_CPL_COMPLETE_FIELD     , "CompleteField"
};

static TOKEN_TABLE  RPN_AttribPosition_Tokens[] = {
    RPN_AT_POS_UNSPECIFIED   , NULL,
    RPN_AT_POS_FIRST_FIELD   , "FirstInField",
    RPN_AT_POS_FIRST_SUBFIELD, "FirstInSubfield",
    RPN_AT_POS_ANYWHERE      , "Anywhere"
};

static TOKEN_TABLE  RPN_AttribRelation_Tokens[] = {
    RPN_AT_REL_LESS_THAN    , "<",
    RPN_AT_REL_LESS_EQUAL   , "<=",
    RPN_AT_REL_EQUAL        , "=",
    RPN_AT_REL_GREATER_EQUAL, ">=",
    RPN_AT_REL_GREATER_THAN , ">",
    RPN_AT_REL_NOT_EQUAL    , "<>"
};

static TOKEN_TABLE  RPN_AttribStruct_Tokens[] = {
    RPN_AT_STR_UNSPECIFIED, NULL,
    RPN_AT_STR_PHRASE     , "Phrase",
    RPN_AT_STR_WORD       , "Word",
    RPN_AT_STR_KEY        , "Key",
    RPN_AT_STR_YEAR       , "Year",
    RPN_AT_STR_DATE_NORM  , "Date",
    RPN_AT_STR_WORD_LIST  , "WordList",
    RPN_AT_STR_DATE_UNNORM, "DateUnnormalized",
    RPN_AT_STR_NAME_NORM  , "Name",
    RPN_AT_STR_NAME_UNNORM, "NameUnnormalized"
};

static TOKEN_TABLE  RPN_AttribUse_Tokens[] = {
    RPN_AT_USE_PERSONAL_NAME  , "PersonalName",
    RPN_AT_USE_CORPORATE_NAME , "CorporateName",
    RPN_AT_USE_CONFERENCE_NAME, "ConferenceName",
    RPN_AT_USE_TITLE          , "Title",
    RPN_AT_USE_SERIES_TITLE   , "SeriesTitle",
    RPN_AT_USE_UNIFORM_TITLE  , "UniformTitle",
    RPN_AT_USE_ISBN           , "ISBN",
    RPN_AT_USE_ISSN           , "ISSN",
    RPN_AT_USE_LC_CARDNUM     , "LCCardNumber",
    RPN_AT_USE_BNB_CARDNUM    , "BNBCardNumber",
    RPN_AT_USE_BGF_NUM        , "BGFNumber",
    RPN_AT_USE_LOCAL_NUM      , "LocalNumber",
    RPN_AT_USE_DEWEY_CLASS    , "DeweyClassification",
    RPN_AT_USE_UDC_CLASS      , "UDCClassification",
    RPN_AT_USE_BLISS_CLASS    , "BlissClassification",
    RPN_AT_USE_LC_CALL_NUM    , "LCCallNumber",
    RPN_AT_USE_NLM_CALL_NUM   , "NLMCallNumber",
    RPN_AT_USE_NAL_CALL_NUM   , "NALCallNumber",
    RPN_AT_USE_MOS_CALL_NUM   , "MOSCallNumber",
    RPN_AT_USE_LOCAL_CLASS    , "LocalClassification",
    RPN_AT_USE_SUBJECT_HEAD   , "SubjectHeading",
    RPN_AT_USE_SUBJECT_RAMEAU , "SubjectRameau",
    RPN_AT_USE_BDI_INDEX_SUB  , "BDIIndexSubject",
    RPN_AT_USE_INSPEC_SUB     , "INSPECSubject",
    RPN_AT_USE_MESH_SUB       , "MESHSubject",
    RPN_AT_USE_PA_SUB         , "PASubject",
    RPN_AT_USE_LC_SUB_HEAD    , "LCSubjectHeading",
    RPN_AT_USE_RVM_SUB_HEAD   , "RVMSubjectHeading",
    RPN_AT_USE_LOCAL_SUB_IDX  , "LocalSubjectIndex",
    RPN_AT_USE_DATE           , "Date",
    RPN_AT_USE_DATE_OF_PUB    , "DateOfPublication",
    RPN_AT_USE_DATE_OF_AQU    , "DateOfAquisition",
    RPN_AT_USE_TITLE_KEY      , "TitleKey",
    RPN_AT_USE_TITLE_COLLECT  , "TitleCollective",
    RPN_AT_USE_TITLE_PARALLEL , "TitleParallel",
    RPN_AT_USE_TITLE_COVER    , "TitleCover",
    RPN_AT_USE_TITLE_ADDEDPG  , "TitleAddedTitlePage",
    RPN_AT_USE_TITLE_CAPTION  , "TitleCaption",
    RPN_AT_USE_TITLE_RUNNING  , "TitleRunning",
    RPN_AT_USE_TITLE_SPINE    , "TitleSpine",
    RPN_AT_USE_TITLE_OTH_VAR  , "TitleOtherVariant",
    RPN_AT_USE_TITLE_FORMER   , "TitleFormer",
    RPN_AT_USE_TITLE_ABBERV   , "TitleAbbreviated",
    RPN_AT_USE_TITLE_EXPANDED , "TitleExpanded",
    RPN_AT_USE_SUB_PRECIS     , "SubjectPrecis",
    RPN_AT_USE_SUB_RSWK       , "SubjectRSWK",
    RPN_AT_USE_SUB_SUBDIVISION, "SubjectSubdivision",
    RPN_AT_USE_NUM_NATL_BIB   , "NumberNatlBibliography",
    RPN_AT_USE_NUM_LEGAL_DEP  , "NumberLegalDeposit",
    RPN_AT_USE_NUM_GOVT_PUB   , "NumberGovtPublication",
    RPN_AT_USE_NUM_PUB_FOR_MUSC,"NumberPubOfMusic",
    RPN_AT_USE_NUM_DB         , "NumberDb",
    RPN_AT_USE_NUM_LOCAL_CALL , "NumberLocalCall",
    RPN_AT_USE_CODE_LANG      , "CodeLanguage",
    RPN_AT_USE_CODE_GEO_AREA  , "CodeGeographicArea",
    RPN_AT_USE_CODE_INSTIT    , "CodeInstitution",
    RPN_AT_USE_NAME_AND_TITLE , "NameAndTitle",
    RPN_AT_USE_NAME_GEOGRAPH  , "NameGeographic",
    RPN_AT_USE_PLACE_PUB      , "PlacePublication",
    RPN_AT_USE_CODEN          , "CODEN",
    RPN_AT_USE_MICROFROM_GEN  , "MicroformGeneration",
    RPN_AT_USE_ABSTRACT       , "Abstract",
    RPN_AT_USE_NOTE           , "Note",
    RPN_AT_USE_AUTHOR_TITLE   , "AuthorTitle",
    RPN_AT_USE_RECORD_TYPE    , "RecordType",
    RPN_AT_USE_NAME           , "Name",
    RPN_AT_USE_AUTHOR         , "Author",
    RPN_AT_USE_AUTHOR_PERSONAL, "AuthorNamePersonal",
    RPN_AT_USE_AUTHOR_CORP    , "AuthorNameCorporation",
    RPN_AT_USE_AUTHOR_CONF    , "AuthorNameConference",
    RPN_AT_USE_ID_STANDARD    , "IdentifierStandard",
    RPN_AT_USE_SUB_LC_CHILD   , "SubjectNameLCChildrens",
    RPN_AT_USE_SUB_NAME_PERS  , "SubjectNamePersonal",
    RPN_AT_USE_BODY_OF_TEXT   , "BodyOfText",
    RPN_AT_USE_DATE_TIME_ADDED, "DateTimeAddedToDatabase",
    RPN_AT_USE_DATE_TIME_MOD  , "DateTimeLastModified",
    RPN_AT_USE_AUTH_FMT_ID    , "AuthorityAndFormatIdentifier",
    RPN_AT_USE_CONCEPT_TEXT   , "ConceptText",
    RPN_AT_USE_CONCEPT_REFER  , "ConceptReference",
    RPN_AT_USE_ANY  	      , "Any",
    RPN_AT_USE_DEFAULT 	      , "Default"
};

static TOKEN_TABLE  RPN_AttribTrunc_Tokens[] = {
    RPN_AT_TRN_UNSPECIFIED, NULL,
    RPN_AT_TRN_RIGHT      , "TruncRight",
    RPN_AT_TRN_LEFT               , "TruncLeft",
    RPN_AT_TRN_BOTH               , "Trunc",
    RPN_AT_TRN_DONT               , "TruncNone",
    RPN_AT_TRN_PROCNUM    , "TruncProcessNum"
};

static TOKEN_TABLE      RPN_Operator_Tokens[] = {
	RPNOP_AND,         "AND",
	RPNOP_OR,          "OR",
	RPNOP_AND_NOT, "AND-NOT"
};

/*
 * UASC list build commands.
 */

void UaSc_ADTCreateList(SLIST structList, SLIST fieldList)
{
	int counter;
	int max;


	max =  table_size(RPN_AttribUse_Tokens);
	
	for (counter = 0; counter < max; counter ++)
	{
		if (RPN_AttribUse_Tokens[counter].name != (char *)NULL)
		{
			(void)slist_add_sorted(fieldList, 
			    (char*)RPN_AttribUse_Tokens[counter].name,
			    (long)RPN_AttribUse_Tokens[counter].value,
			    FALSE, FALSE);
		}
		else
		{
			(void)slist_add_sorted(fieldList, 
			    (char*)"None",
			    (long)RPN_AttribUse_Tokens[counter].value,
			    FALSE, FALSE);
		}
	}

	max = table_size(RPN_AttribStruct_Tokens);
	
	for (counter = 0; counter < max; counter++)
	{
		if (RPN_AttribStruct_Tokens[counter].name != (char *)NULL)
		{
			(void)slist_add_sorted(structList, 
			    (char*)RPN_AttribStruct_Tokens[counter].name,
			    (long)RPN_AttribStruct_Tokens[counter].value,
			    FALSE, FALSE);
		}
		else
		{
			(void)slist_add_sorted(structList, 
			    (char*)"None",
			    (long)RPN_AttribStruct_Tokens[counter].value,
			    FALSE, FALSE);
		}
	}
}

static int      RPN_Structure_TreePrintAux(RPN_Structure *, int (*item_print)(const char *, int, int, long), long, int);


static char *token_lookup(
    TOKEN_TABLE *table,
    size_t              table_size,
    int                 value
    )
{
	for ( ; table_size-- > 0; table++)
		if (table->value == value)
			return table->name;

    return NULL;
}


static char *token_lookup_strdup(
    TOKEN_TABLE *table,
    size_t              table_size,
    int                 value
    )
{
    char    *s;

    if (s = token_lookup(table, table_size, value))
		return strdup(s);
    return NULL;
}


void RPN_Attribs_List(
	int     (*add_item)(const char *item_name, RPN_AttribType type, int value, long user_data),
	long    user_data
	)
{
	int     i, j;
	static struct {
		TOKEN_TABLE             *table;
		RPN_AttribType  type;
		int                     item_count;
	} tables[] = {
		RPN_AttribUse_Tokens     , RPN_AT_USE             , table_size(RPN_AttribUse_Tokens),
		RPN_AttribRelation_Tokens, RPN_AT_RELATION        , table_size(RPN_AttribRelation_Tokens),
		RPN_AttribPosition_Tokens, RPN_AT_POSITION        , table_size(RPN_AttribPosition_Tokens),
		RPN_AttribStruct_Tokens  , RPN_AT_STRUCTURE       , table_size(RPN_AttribStruct_Tokens),
		RPN_AttribTrunc_Tokens   , RPN_AT_TRUNCATION  , table_size(RPN_AttribTrunc_Tokens),
		RPN_AttribComplete_Tokens, RPN_AT_COMPLETENESS, table_size(RPN_AttribComplete_Tokens)
	};

	for (i = 0; i < table_size(tables); i++)
	{
		for (j = 0; j < tables[i].item_count; j++)
		{
			if ((*add_item)(tables[i].table[j].name,
			    tables[i].type,
			    tables[i].table[j].value,
			    user_data))
			{
				return;
			}
		}
	}
}


RPN_ResultSetId *RPN_ResultSetId_Copy(
    RPN_ResultSetId *rid
    )
{
    if (!rid)
		return (RPN_ResultSetId *)0;

    return RPN_ResultSetId_New(rid->set_name);
}


void RPN_ResultSetId_Free(
    RPN_ResultSetId *rid
    )
{
    if (rid) {
		if (rid->set_name)
			RescIR_Free(IR_FILE, rid->set_name);
		RescIR_Free(IR_FILE, (char *)rid);
    }
}


RPN_ResultSetId *RPN_ResultSetId_New(
    const char  *result_set_name
    )
{
    RPN_ResultSetId *val;

    if (!result_set_name)
		return (RPN_ResultSetId *)0;

	if (val = new_struct(RPN_ResultSetId))
		val->set_name = strdup((char *)result_set_name);

    return val;
}


char *RPN_AttribUse_Print(
    RPN_AttribUse   val
    )
{
	char *s;
    s = token_lookup_strdup(RPN_AttribUse_Tokens, table_size(RPN_AttribUse_Tokens), (int)val);
	return(s);
}


char *RPN_AttribRelation_Print(
    RPN_AttribRelation  val
    )
{
	char *s;
    s = token_lookup_strdup(RPN_AttribRelation_Tokens, table_size(RPN_AttribRelation_Tokens), (int)val);
	if (s == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "RPN_AttribRelation_Tokens returned null\n");
	}
	return(s);
}


char *RPN_AttribPosition_Print(
    RPN_AttribPosition  val
    )
{
	char *s;
    s = token_lookup_strdup(RPN_AttribPosition_Tokens, table_size(RPN_AttribPosition_Tokens), (int)val);
	if (s == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "RPN_AttribPosition_Tokens returned null\n");
	}
	return(s);
}


char *RPN_AttribStruct_Print(
    RPN_AttribStruct    val
    )
{
	char *s;
    s = token_lookup_strdup(RPN_AttribStruct_Tokens, table_size(RPN_AttribStruct_Tokens), (int)val);
	if (s == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "token_look_up returned null\n");
	}
	return(s);
}


char *RPN_AttribTrunc_Print(
    RPN_AttribTrunc val
    )
{
	char *s;
    s = token_lookup_strdup(RPN_AttribTrunc_Tokens, table_size(RPN_AttribTrunc_Tokens), (int)val);
	if (s == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "token_look_up returned null\n");
	}
	return(s);
}


char *RPN_AttribComplete_Print(
    RPN_AttribComplete  val
    )
{
	char *s;
    s = token_lookup_strdup(RPN_AttribComplete_Tokens, table_size(RPN_AttribComplete_Tokens), (int)val);
	if (s == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "token_look_up returned null\n");
	}
	return(s);
}


RPN_AttribTerm *RPN_AttribTerm_Copy(
    RPN_AttribTerm  *at
    )
{
    if (!at)
		return (RPN_AttribTerm *)0;

    return RPN_AttribTerm_New(
	at->use,
	at->relation,
	at->position,
	at->structure,
	at->truncation,
	at->completeness,
	at->value,
	at->class,
	at->className,
	at->classIndex);
}


RPN_AttribTerm *RPN_AttribTerm_New(
    RPN_AttribUse       use,
    RPN_AttribRelation  relation,
    RPN_AttribPosition  position,
    RPN_AttribStruct    structure,
    RPN_AttribTrunc     truncation,
    RPN_AttribComplete  completeness,
    char                *value,
    RPN_ATTRIB_CLASS    class,
    char                *className,
    int                 classIndex)
{
	RPN_AttribTerm  *val;

	if (val = new_struct(RPN_AttribTerm))
	{
		val->use          = use;
		val->relation     = relation;
		val->position     = position;
		val->structure    = structure;
		val->truncation   = truncation;
		val->completeness = completeness;
		val->value        = value ? strdup(value) : NULL;
		val->class        = class;
		val->className    = className ? strdup(className) : NULL;
		val->classIndex   = classIndex;
	}

	return val;
}


void RPN_AttribTerm_Free(
    RPN_AttribTerm  *at
    )
{
    	if (at) 
	{
		if (at->value)
		{
			RescIR_Free(IR_FILE, at->value);
		}
		if (at->className)
		{
			RescIR_Free(IR_FILE, at->className);
		}
		
		RescIR_Free(IR_FILE, (char *)at);

	}
}


char *RPN_AttribTerm_Print(
    RPN_AttribTerm  *at)
{
	char    buf[BUFSIZ];
	char    *funcs[4];
	int     i;
	char    *rel;
	char    *use;
	char    valbuf[BUFSIZ];

	if (at->class == RPN_LOCAL_DEFINE)
	{
		funcs[0] = RPN_AttribPosition_Print(at->position);
		funcs[1] = RPN_AttribStruct_Print(at->structure);
		funcs[2] = RPN_AttribTrunc_Print(at->truncation);
		funcs[3] = RPN_AttribComplete_Print(at->completeness);

		sprintf(valbuf, "\"%s\"", at->value ? at->value : "");

		for (i = 0; i < sizeof(funcs) / sizeof(funcs[0]); i++)
		 {
			if (funcs[i])
			{
				sprintf(buf, "%s(%s)", funcs[i], valbuf);
				RescIR_Free(IR_FILE, funcs[i]);
			}
		}

		sprintf(buf, "%s %s %s",
		    use = RPN_AttribUse_Print(at->use),
		    rel = RPN_AttribRelation_Print(at->relation),
		    valbuf);
		RescIR_Free(IR_FILE, use);
	}
	else
	{
		sprintf(valbuf, "\"%s\"", at->value ? at->value : "");
		sprintf(buf, "%s %s %s",
		    at->className,
		    rel = RPN_AttribRelation_Print(at->relation),
		    valbuf);
	}

	RescIR_Free(IR_FILE, rel);

	return strdup(buf);
}


RPN_Operand *RPN_Operand_Copy(
    RPN_Operand *op
    )
{
    if (!op)
		return (RPN_Operand *)0;

    if (op->rpn_op_type == RPN_OPERAND_ATTRIB_TERM)
		return RPN_Operand_NewAT(op->v.rpn_op_at);
    if (op->rpn_op_type == RPN_OPERAND_RESULT_SET)
		return RPN_Operand_NewRS(op->v.rpn_op_rid);

    return (RPN_Operand *)0;
}


RPN_Operand *RPN_Operand_NewAT(
    RPN_AttribTerm  *at
    )
{
    RPN_Operand *val;

    if (!at)
		return (RPN_Operand *)0;

	if (val = new_struct(RPN_Operand)) {
		val->rpn_op_type = RPN_OPERAND_ATTRIB_TERM;
		val->v.rpn_op_at = RPN_AttribTerm_Copy(at);
    }

    return val;
}


RPN_Operand *RPN_Operand_NewRS(
    RPN_ResultSetId  *rid
    )
{
    RPN_Operand *val;

    if (!rid)
		return (RPN_Operand *)0;

	if (val = new_struct(RPN_Operand)) {
		val->rpn_op_type  = RPN_OPERAND_RESULT_SET;
		val->v.rpn_op_rid = RPN_ResultSetId_Copy(rid);
    }

    return val;
}


void RPN_Operand_Free(
    RPN_Operand *op
    )
{
    if (op) {
		if (op->rpn_op_type == RPN_OPERAND_ATTRIB_TERM)
			RPN_AttribTerm_Free(op->v.rpn_op_at);
		if (op->rpn_op_type == RPN_OPERAND_RESULT_SET)
			RPN_ResultSetId_Free(op->v.rpn_op_rid);

		RescIR_Free(IR_FILE, (char *)op);
    }
}


char *RPN_Operand_Print(
    RPN_Operand *op
    )
{
	char *s;
    if (!op)
	{
	CS_LOG_PRINTS(IR_FILE, "no operand\n");
		return NULL;
	}

    if (op->rpn_op_type == RPN_OPERAND_ATTRIB_TERM)
	{
		s = RPN_AttribTerm_Print(op->v.rpn_op_at);
		return(s);
	}
    if (op->rpn_op_type == RPN_OPERAND_RESULT_SET)
	{
		return NULL;
	}
CS_LOG_PRINTS(IR_FILE, "Bad RPN operand\n");
    return strdup("*** Bad RPN_Operand ***");
}


char *RPN_Operator_Print(
	RPN_Operator    val
    )
{
	char *s;
	s = token_lookup_strdup(RPN_Operator_Tokens, table_size(RPN_Operator_Tokens), (int)val);
	if (s == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "token look up returned null\n");
	}
	return(s);
}


RPN_Expression *RPN_Expression_Copy(
    RPN_Expression  *ex
    )
{
    if (!ex)
	{
CS_LOG_PRINTS(IR_FILE, "no expression\n");
		return (RPN_Expression *)0;
	}

    return RPN_Expression_New(ex->op1, ex->operator, ex->op2);
}


void RPN_Expression_Free(
    RPN_Expression  *ex
    )
{
    if (ex) {
		RPN_Structure_Free(ex->op1);
		RPN_Structure_Free(ex->op2);
		RescIR_Free(IR_FILE, (char *)ex);
    }
}


RPN_Expression *RPN_Expression_New(
    struct _RPN_Structure   *op1,
    RPN_Operator                    operator,
    struct _RPN_Structure   *op2
    )
{
    RPN_Expression  *val;

    if (!op1  ||  !op2)
		return (RPN_Expression *)0;

	if (val = new_struct(RPN_Expression)) {
		val->op1      = RPN_Structure_Copy(op1);
		val->operator = operator;
		val->op2      = RPN_Structure_Copy(op2);
    }

    return val;
}


char *RPN_Expression_Print(
    RPN_Expression  *expr)
{
	char    buf[BUFSIZ];
	char    buf_op1[BUFSIZ];
	char    buf_op2[BUFSIZ];
	char    *s;

	if (!expr)
	{
		return NULL;
	}

	s = RPN_Structure_Print(expr->op1);
	if (expr->op1->rpn_struct_type == RPN_STRUCT_EXPR)
	{
		sprintf(buf_op1, "(%s)", s);
	}
	else
	{
		strcpy(buf_op1, s);
	}
	RescIR_Free(IR_FILE, s);


	s = RPN_Structure_Print(expr->op2);
	if (expr->op2->rpn_struct_type == RPN_STRUCT_EXPR)
	{
		sprintf(buf_op2, "(%s)", s);
	}
	else
	{
		strcpy(buf_op2, s);
	}
	RescIR_Free(IR_FILE, s);

	s = RPN_Operator_Print(expr->operator);

	sprintf(buf, "%s %s %s", buf_op1, s, buf_op2);
	RescIR_Free(IR_FILE, s);

	return strdup(buf);
}


int RPN_Expression_TreePrint(
    RPN_Expression  *expr,
    int (*item_print)(const char *item, int indent_level, int is_operand, long user_data),
    long user_data,
    int indent_level)
{
	char *s;
	int  val;

	if (!expr)
	{
		return 0;
	}

	/*
	 * Print the first operand.
	 */
	if (val = RPN_Structure_TreePrintAux(expr->op1,
	    item_print,
	    user_data,
	    (indent_level + 1)))
	{
		return val;
	}

	/*
	 * Print the operator.
	 */
	s = RPN_Operator_Print(expr->operator);
	if (val = (*item_print)(s, indent_level, FALSE, user_data))
	{
		RescIR_Free(IR_FILE, s);
		return val;
	}
	RescIR_Free(IR_FILE, s);

	/*
	 * Print the second Operand.
	 */
	return RPN_Structure_TreePrintAux(expr->op2,
	    item_print,
	    user_data,
	    (indent_level + 1));
}


void RPN_Structure_Free(
    RPN_Structure   *str
    )
{
    if (str) {
		if (str->rpn_struct_type == RPN_STRUCT_OPERAND)
			RPN_Operand_Free(str->v.operand);
		else if (str->rpn_struct_type == RPN_STRUCT_EXPR)
			RPN_Expression_Free(str->v.expression);

		RescIR_Free(IR_FILE, (char *)str);
    }
}


RPN_Structure *RPN_Structure_Copy(
    RPN_Structure   *str
    )
{
    if (!str)
		return (RPN_Structure *)0;

    if (str->rpn_struct_type == RPN_STRUCT_OPERAND)
		return RPN_Structure_NewOp(str->v.operand);
    if (str->rpn_struct_type == RPN_STRUCT_EXPR)
		return RPN_Structure_NewEx(str->v.expression);

    return (RPN_Structure *)0;
}


RPN_Structure *RPN_Structure_NewOp(
    RPN_Operand *op
    )
{
    RPN_Structure   *val;

    if (!op)
		return (RPN_Structure *)0;

	if (val = new_struct(RPN_Structure)) {
		val->rpn_struct_type = RPN_STRUCT_OPERAND;
		val->v.operand           = RPN_Operand_Copy(op);
    }

    return val;
}


RPN_Structure *RPN_Structure_NewEx(
    RPN_Expression  *ex
    )
{
    RPN_Structure   *val;

    if (!ex)
		return (RPN_Structure *)0;

	if (val = new_struct(RPN_Structure)) {
		val->rpn_struct_type = RPN_STRUCT_EXPR;
		val->v.expression        = RPN_Expression_Copy(ex);
    }

    return val;
}


char *RPN_Structure_Print(
    RPN_Structure   *str
    )
{
	char *s;
    if (!str)
		return NULL;

    if (str->rpn_struct_type == RPN_STRUCT_OPERAND)
	{
		s = RPN_Operand_Print(str->v.operand);
		return(s);
	}
    if (str->rpn_struct_type == RPN_STRUCT_EXPR)
	{
		s = RPN_Expression_Print(str->v.expression);
		return(s);
	}

CS_LOG_PRINTS(IR_FILE, "bad rpn structure\n");
    return strdup("*** Bad RPN_Structure ***");
}


static int RPN_Structure_TreePrintAux(
    RPN_Structure   *str,
	int                             (*item_print)(const char *item, int indent_level, int is_operand, long user_data),
	long                    user_data,
    int                     indent_level
    )
{
    char    *s;
    int     val;

    if (!str)
		return 0;

    if (str->rpn_struct_type == RPN_STRUCT_OPERAND) {
		s = RPN_Operand_Print(str->v.operand);
		val = (*item_print)(s, indent_level, TRUE, user_data);
		RescIR_Free(IR_FILE, s);
		return val;
    }

    if (str->rpn_struct_type == RPN_STRUCT_EXPR)
		return RPN_Expression_TreePrint(str->v.expression, item_print, user_data, indent_level);

    return -1;
}


int RPN_Structure_TreePrint(
    RPN_Structure   *str,
	int                             (*item_print)(const char *item, int indent_level, int is_operand, long user_data),
	long                    user_data
    )
{
	return RPN_Structure_TreePrintAux(str, item_print, user_data, 0);
}


static RPN_Structure *RPN_Structure_TreeFindAux(
    RPN_Structure   *str,         /* In: tree to search */
    RPN_Structure   *str_parent,  /* In: parent tree to search */
    int             *print_order, /* In/Out: What order was the item in the TreePrint, starting at 0 */
    RPN_Structure   **parent)     /* Out: Parent of the found node.  Null if node is top-level entry */
				  /* Returns: The found node, Null if not found */
{
	RPN_Structure   *ret_val;

	if (!str)
	{
		return str;
	}

	if (str->rpn_struct_type == RPN_STRUCT_EXPR)
	{
		if (ret_val = RPN_Structure_TreeFindAux(
		    str->v.expression->op1,
		    str,
		    print_order,
		    parent))
		{
			return ret_val;
		}

		if ((*print_order) <= 0)
		{
			/*
			 * The item they want is the operator
			 * of the expression
			 */
			*parent = str_parent;
			return str;
		}
		(*print_order)--;

		return (RPN_Structure_TreeFindAux(
		    str->v.expression->op2,
		    str,
		    print_order,
		    parent));
	}

	if (str->rpn_struct_type == RPN_STRUCT_OPERAND)
	{
		if (*print_order <= 0)
		{
		    *parent = str_parent;
		    return str;
		}
		(*print_order)--;
	}

	return (RPN_Structure *)0;      /* Didn't find it in this subtree */
}


RPN_Structure *RPN_Structure_TreeFind(
    RPN_Structure   *str,           /* In: tree to search */
    int             print_order,    /* In: What order was the item in the TreePrint, starting at 0 */
    RPN_Structure   **parent        /* Out: Parent of the found node.  Null if node is top-level entry */
    )                               /* Returns: The found node, Null if not found */
{
    print_order--;
    return RPN_Structure_TreeFindAux(str, (RPN_Structure *)0, &print_order, parent);
}

int RPN_Structure_SizeAux(RPN_Structure *str, int size)
{
	if (!str)
	{
		return size;
	}
	/*
	 * Count this node.
	 */
	size++;

	if (str->rpn_struct_type == RPN_STRUCT_EXPR)
	{
		/*
		 * Count the right branch.
		 */
		size = RPN_Structure_SizeAux(
		    str->v.expression->op1,
		    size);
		/*
		 * Count the left branch.
		 */
		size = RPN_Structure_SizeAux(
		    str->v.expression->op2,
		    size);
	}
	/*
	 * Return the size.
	 */
	return size;
}


int RPN_Structure_Size(RPN_Structure   *str)
{
	return RPN_Structure_SizeAux(str, 0);
}

UaSc_QFreeRPNStructure()
{
	xvt_note("This is the Q RPN Query free routine. Who called?");
	return(0);
}


void UaSc_GenRPNExpression(IR_RPN_ELEMENT *elem, RPN_Expression *exp);
void UaSc_GenRPNOperand(IR_RPN_ELEMENT *elem, RPN_Operand *operand);
void RPN_StructToIrRPN(RPN_Structure *str, IR_RPN_QUERY_STRUCT *query)
{
	RESC_ASN_NO_MALLOC(query);
	RESC_ASN_NO_MALLOC(&query->attributeSetId);
	RESC_ASN_INIT_OI(&query->attributeSetId);
	RescASN_AddOI(IR_FILE, &query->attributeSetId,
	    OID_ATTRIBUTE_SET_BIB1, strlen(OID_ATTRIBUTE_SET_BIB1));

	/* 
	 * Start with first element and perform a recursive conversion
	 * of all elements in the binary tree.
	 */
	RESC_ASN_NO_MALLOC(&query->rpnElem);
	if (str->rpn_struct_type == RPN_STRUCT_OPERAND)
	{
		/* Convert an operand only */
		query->rpnElem.rpnOperand = NULL;
		UaSc_GenRPNOperand(&query->rpnElem, str->v.operand);
	}
	else
	{
		/* Convert the complex expression */
		query->rpnElem.rpnStructSeq = NULL;
		UaSc_GenRPNExpression(&query->rpnElem, str->v.expression);
	}
}

IR_ATTR_ELEM    *UaSc_GenAttrElem(CHAR *src)
{
	IR_ATTR_ELEM    *new;

	new = (IR_ATTR_ELEM *)RescIR_Zmalloc(IR_FILE, 
	    sizeof(IR_ATTR_ELEM));
	if (new == NULL)
	{
		return(NULL);
	}
	RESC_ASN_MALLOCED(new);
	return(new);
}

void UaSc_DestroyAttrElem(CHAR *src, void *data)
{
	IR_ATTR_ELEM    *attr;

	if (data == NULL)
	{
		return;
	}

	attr = (IR_ATTR_ELEM *)data;

	if (RESC_ASN_IS_MALLOCED(attr))
	{
		RescIR_Free(src, (CHAR *)attr);
	}
}
		
IR_ATTR_ELEM    *UaSc_CloneAttrElem(CHAR *src, void *data)
{
	IR_ATTR_ELEM    *attr;
	IR_ATTR_ELEM    *new;

	if (data == NULL)
	{
		return(NULL);
	}

	attr = (IR_ATTR_ELEM *)data;

	new = UaSc_GenAttrElem(src);

	new->attrType = attr->attrType;
	new->attrValue = attr->attrValue;

	return(new);
}

void UaSc_GenRPNOperand(IR_RPN_ELEMENT *elem, RPN_Operand *operand)
{
	RPN_AttribTerm  *attrTerm;
	IR_ATTR_ELEM    *attr;
	elem->rpnElemType = IR_RPN_NODE_SIMPLE;

	if (elem->rpnOperand == NULL)
	{
		elem->rpnOperand = (IR_RPN_SIMPLE_NODE *)RescIR_Zmalloc(
		    IR_FILE, sizeof(IR_RPN_SIMPLE_NODE));

		if (elem == NULL)
		{
			CS_LOG_PRINTS(IR_FILE, "Memory error\n");
			return;
		}
		RESC_ASN_MALLOCED(elem->rpnOperand);
		RESC_ASN_NO_MALLOC(&elem->rpnOperand->resultSetId);
		RESC_ASN_NO_MALLOC(&elem->rpnOperand->attrPlusTerm);
		RESC_ASN_NO_MALLOC(&elem->rpnOperand->attrPlusTerm.attrList);
		RESC_LIST_INIT(&elem->rpnOperand->attrPlusTerm.attrList);
		RESC_ASN_NO_MALLOC(&elem->rpnOperand->attrPlusTerm.term);
		RESC_ASN_INIT_OS(&elem->rpnOperand->attrPlusTerm.term);
	}

	switch(operand->rpn_op_type)
	{
	case RPN_OPERAND_ATTRIB_TERM:
		elem->rpnOperand->simpleType = IR_RPN_SIMPLE_ATTR;
		/* Copy the term */
		attrTerm = operand->v.rpn_op_at;
		RescASN_AddOS(IR_FILE, &elem->rpnOperand->attrPlusTerm.term,
		    attrTerm->value, strlen(attrTerm->value));
		
		attr = UaSc_GenAttrElem(IR_FILE);

		attr->attrValue = attrTerm->use;
		attr->attrType = 1;

		RescList_Append(IR_FILE, 
		    &elem->rpnOperand->attrPlusTerm.attrList,
		    attr, UaSc_DestroyAttrElem, UaSc_CloneAttrElem);

		attr = UaSc_GenAttrElem(IR_FILE);

		attr->attrValue = attrTerm->relation;
		attr->attrType = 2;

		RescList_Append(IR_FILE, 
		    &elem->rpnOperand->attrPlusTerm.attrList,
		    attr, UaSc_DestroyAttrElem, UaSc_CloneAttrElem);

		if (attrTerm->position != RPN_AT_POS_UNSPECIFIED)
		{
			attr = UaSc_GenAttrElem(IR_FILE);

			attr->attrValue = attrTerm->position;
			attr->attrType = 3;

			RescList_Append(IR_FILE, 
			    &elem->rpnOperand->attrPlusTerm.attrList,
			    attr, UaSc_DestroyAttrElem, UaSc_CloneAttrElem);
		}
		
		if (attrTerm->structure != RPN_AT_STR_UNSPECIFIED)
		{
			attr = UaSc_GenAttrElem(IR_FILE);

			attr->attrValue = attrTerm->structure;
			attr->attrType = 4;

			RescList_Append(IR_FILE, 
			    &elem->rpnOperand->attrPlusTerm.attrList,
			    attr, UaSc_DestroyAttrElem, UaSc_CloneAttrElem);
		}

		if (attrTerm->truncation != RPN_AT_TRN_UNSPECIFIED)
		{
			attr = UaSc_GenAttrElem(IR_FILE);

			attr->attrValue = attrTerm->truncation;
			attr->attrType = 5;

			RescList_Append(IR_FILE, 
			    &elem->rpnOperand->attrPlusTerm.attrList,
			    attr, UaSc_DestroyAttrElem, UaSc_CloneAttrElem);
		}

		if (attrTerm->completeness != RPN_AT_CPL_UNSPECIFIED)
		{
			attr = UaSc_GenAttrElem(IR_FILE);

			attr->attrValue = attrTerm->completeness;
			attr->attrType = 6;

			RescList_Append(IR_FILE, 
			    &elem->rpnOperand->attrPlusTerm.attrList,
			    attr, UaSc_DestroyAttrElem, UaSc_CloneAttrElem);
		}

		break;
	case RPN_OPERAND_RESULT_SET:
		elem->rpnOperand->simpleType = IR_RPN_SIMPLE_RESULT_SET_ID;
		/* Copy the result set Id */
		RescASN_AddOS(IR_FILE, &elem->rpnOperand->resultSetId,
		    operand->v.rpn_op_rid->set_name,
		    strlen(operand->v.rpn_op_rid->set_name));
		
		break;
	default:
		break;
	}
}

void UaSc_GenRPNExpression(IR_RPN_ELEMENT *elem, RPN_Expression *exp)
{
	IR_RPN_COMPLEX_NODE     *newNode;
	IR_RPN_ELEMENT          *newElem;
	RPN_Structure           *nextExp;

	elem->rpnElemType = IR_RPN_NODE_COMPLEX;

	if (elem->rpnStructSeq == NULL)
	{
		elem->rpnStructSeq = (IR_RPN_COMPLEX_NODE *)RescIR_Zmalloc(
		    IR_FILE, sizeof(IR_RPN_COMPLEX_NODE));
		if (elem->rpnStructSeq == NULL)
		{
			CS_LOG_PRINTS(IR_FILE, 
			    "Memory error mallocing complex node \n");
			return;
		}
		RESC_ASN_MALLOCED(elem->rpnStructSeq);
	}

	newNode = elem->rpnStructSeq;
	switch(exp->operator)
	{
	case RPNOP_AND:
		newNode->operator = 0; 
		break;
	case RPNOP_OR:
		newNode->operator = 1;
		break;
	case RPNOP_AND_NOT:
		newNode->operator = 2;
		break;
	default:
		break;
	}

	newElem = (IR_RPN_ELEMENT *)RescIR_Zmalloc(IR_FILE,
	    sizeof(IR_RPN_ELEMENT));
	if (newElem == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Error mallocing first exp elem\n");
		return;
	}
	newNode->rpnElem1 = (struct IR_RPN_ELEMENT *)newElem;
	nextExp = exp->op1;

	if (nextExp->rpn_struct_type == RPN_STRUCT_OPERAND)
	{
		UaSc_GenRPNOperand(newElem, nextExp->v.operand);
	}
	else
	{
		UaSc_GenRPNExpression(newElem, nextExp->v.expression);
	}
	
	newElem = (IR_RPN_ELEMENT *)RescIR_Zmalloc(IR_FILE,
	    sizeof(IR_RPN_ELEMENT));
	if (newElem == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Error mallocing first exp elem\n");
		return;
	}
	newNode->rpnElem2 = (struct IR_RPN_ELEMENT *)newElem;
	nextExp = exp->op2;

	if (nextExp->rpn_struct_type == RPN_STRUCT_OPERAND)
	{
		UaSc_GenRPNOperand(newElem, nextExp->v.operand);
	}
	else
	{
		UaSc_GenRPNExpression(newElem, nextExp->v.expression);
	}
}
