/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

#include <ir.h>
#include <ua.h>
#include <resc.h>
#include "xvt.h"
#include "uaep.h"
#include "query.h"
#include <ua\rpnquery.h>
#include <debug.h>


static SLIST    use_attribs;    /* to the items contained in the dialog list boxes. */

static RPN_AttribTerm  BasicDisplayList[] = {
/*      RPN_AttribUse,    RPN_AttribRelation,    RPN_AttribPosition,
 *         RPN_AttribStruct,   RPN_AttribTrunc, RPN_AttribComplete,
 *         List Display,            RPN_ATTRIB_CLASS,    structure value.
 */
    /*
     * KEYWORD Searches
     */
    RPN_AT_USE_NAME,    RPN_AT_REL_EQUAL,        RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_WORD,   RPN_AT_TRN_UNSPECIFIED,    RPN_AT_CPL_UNSPECIFIED,
    "Name Keywords(NW)",    RPN_PRE_DEFINE,     "NW",  0,

    RPN_AT_USE_AUTHOR,  RPN_AT_REL_EQUAL,        RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_WORD,   RPN_AT_TRN_UNSPECIFIED,    RPN_AT_CPL_UNSPECIFIED,
    "Author Name Keywords(AW)",     RPN_PRE_DEFINE,     "AW",   0,

    RPN_AT_USE_TITLE,   RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_WORD,   RPN_AT_TRN_UNSPECIFIED,    RPN_AT_CPL_UNSPECIFIED,
    "Title Keywords(TW)",   RPN_PRE_DEFINE,     "TW",  0,

    RPN_AT_USE_SUBJECT_HEAD,    RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_WORD,   RPN_AT_TRN_UNSPECIFIED,    RPN_AT_CPL_UNSPECIFIED,
    "Subject Keywords(SW)", RPN_PRE_DEFINE,     "SW",  0,

    /*
     * NAME INDEXES
     */
    RPN_AT_USE_AUTHOR,  RPN_AT_REL_EQUAL,        RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED, RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Author Name(AU)",      RPN_PRE_DEFINE,     "AU",  0,

    RPN_AT_USE_PERSONAL_NAME,   RPN_AT_REL_EQUAL,        RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED, RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Person or Family(PN)", RPN_PRE_DEFINE,     "PN", 0,

    RPN_AT_USE_AUTHOR_PERSONAL, RPN_AT_REL_EQUAL,        RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED, RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Person or Family as Author(AP)",       RPN_PRE_DEFINE,     "AP",  0,

    RPN_AT_USE_CORPORATE_NAME,  RPN_AT_REL_EQUAL,        RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED, RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Corporate body or government(CN)",     RPN_PRE_DEFINE,     "CN",  0,

    RPN_AT_USE_AUTHOR_CORP,     RPN_AT_REL_EQUAL,        RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED, RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Corporate body or government as Author(AC)",   RPN_PRE_DEFINE,   "AC",  0,

    RPN_AT_USE_CONFERENCE_NAME, RPN_AT_REL_EQUAL,        RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED, RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Meeting or conference(MN)",    RPN_PRE_DEFINE,     "MN",  0,

    RPN_AT_USE_AUTHOR_CONF,     RPN_AT_REL_EQUAL,        RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED, RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Meeting or conference as Author(AM)",  RPN_PRE_DEFINE,     "AM",  0,

    RPN_AT_USE_AUTHOR_TITLE,  RPN_AT_REL_EQUAL,        RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED, RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Author Title(AT)",  RPN_PRE_DEFINE,     "AT",  0,

    /*
     * Titles and subject searches
     */
    RPN_AT_USE_TITLE,   RPN_AT_REL_EQUAL,        RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,    RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Title(TI)",    RPN_PRE_DEFINE,     "TI",  0,

    RPN_AT_USE_SERIES_TITLE,    RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,     RPN_AT_TRN_UNSPECIFIED,  RPN_AT_CPL_UNSPECIFIED,
    "Series Title(ST)",  RPN_PRE_DEFINE,     "ST",  0,

    RPN_AT_USE_UNIFORM_TITLE,    RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,     RPN_AT_TRN_UNSPECIFIED,  RPN_AT_CPL_UNSPECIFIED,
    "Series Title(UT)",  RPN_PRE_DEFINE,     "UT",  0,

    RPN_AT_USE_TITLE_FORMER,    RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,     RPN_AT_TRN_UNSPECIFIED,  RPN_AT_CPL_UNSPECIFIED,
    "Former Title(FT)",  RPN_PRE_DEFINE,     "FT",  0,

    RPN_AT_USE_SUB_NAME_PERS,   RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,    RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Person or Family as Subject(SP)",      RPN_PRE_DEFINE,     "SP",  0,

    RPN_AT_USE_SUBJECT_HEAD,    RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,     RPN_AT_TRN_UNSPECIFIED,  RPN_AT_CPL_UNSPECIFIED,
    "Subject(SU)",  RPN_PRE_DEFINE,     "SU",  0,

    RPN_AT_USE_MESH_SUB,    RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,     RPN_AT_TRN_UNSPECIFIED,  RPN_AT_CPL_UNSPECIFIED,
    "MESH Subject(MS)",  RPN_PRE_DEFINE,     "MS",  0,

    RPN_AT_USE_LOCAL_SUB_IDX,    RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,     RPN_AT_TRN_UNSPECIFIED,  RPN_AT_CPL_UNSPECIFIED,
    "Local Subject Index Term(IS)",  RPN_PRE_DEFINE,     "IS",  0,

    /*
     * Date Searches
     */
    RPN_AT_USE_DATE,    RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_WORD, RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Date(DA)",     RPN_PRE_DEFINE,     "DA",  0,

    RPN_AT_USE_DATE_OF_PUB,     RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_WORD, RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Date of Publication(DP)",      RPN_PRE_DEFINE,     "DP",  0,

    RPN_AT_USE_DATE_OF_PUB,     RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_WORD, RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Date of Acquisition(AQ)",      RPN_PRE_DEFINE,     "AQ",  0,

    /*
     * Other Searches
     */
    RPN_AT_USE_CODE_LANG,      RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,    RPN_AT_TRN_UNSPECIFIED,  RPN_AT_CPL_UNSPECIFIED,
    "Language of an Item(LA)",      RPN_PRE_DEFINE,     "LA",  0,

    RPN_AT_USE_PLACE_PUB,       RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,    RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Place of publication(PP)",     RPN_PRE_DEFINE,     "PP",  0,

    RPN_AT_USE_ISBN,    RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,    RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "ISBN(B#)",     RPN_PRE_DEFINE,     "B#",  0,

    RPN_AT_USE_ISSN,    RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,   RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "ISSN(S#)",     RPN_PRE_DEFINE,     "S#",  0,

    RPN_AT_USE_LC_CALL_NUM,    RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,    RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Library of Congress Call Number(LC)",  RPN_PRE_DEFINE,     "LC",  0,

    RPN_AT_USE_LOCAL_NUM,   RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,   RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Local Number(LN)",     RPN_PRE_DEFINE,     "LN",  0,

    RPN_AT_USE_NUM_GOVT_PUB, RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,  RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Government Publication Number(GP)", RPN_PRE_DEFINE,     "GP",  0,

    RPN_AT_USE_NUM_PUB_FOR_MUSC, RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,  RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Music Publisher Number(MP)", RPN_PRE_DEFINE,     "MP",  0,

    RPN_AT_USE_NUM_DB, RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,  RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Database Number(DB)", RPN_PRE_DEFINE,     "DB",  0,

    RPN_AT_USE_LC_CARDNUM,   RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,  RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Library of Congress Card Number(L#)",  RPN_PRE_DEFINE,     "L#",  0,

    RPN_AT_USE_CODEN,   RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,  RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "CODEN(CO)",  RPN_PRE_DEFINE,     "CO",  0,

    RPN_AT_USE_NOTE,   RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,  RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Note (NT)",  RPN_PRE_DEFINE,     "NT#",  0,

    RPN_AT_USE_BODY_OF_TEXT,   RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,  RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Body of Text(BT)",  RPN_PRE_DEFINE,     "BT",  0,

    RPN_AT_USE_ANY,   RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,  RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Any(AN)",  RPN_PRE_DEFINE,     "AN",  0,

    RPN_AT_USE_DEFAULT,   RPN_AT_REL_EQUAL, RPN_AT_POS_UNSPECIFIED,
    RPN_AT_STR_UNSPECIFIED,  RPN_AT_TRN_UNSPECIFIED,   RPN_AT_CPL_UNSPECIFIED,
    "Default(DF)",  RPN_PRE_DEFINE,     "DF",  0
};

static BOOLEAN Selected;
static BOOLEAN ModifyMode;

STATICFCN void show_query XVT_CC_ARGS((WINDOW win, UAEP_DIALOG *info));
STATICFCN int  tree_to_list XVT_CC_ARGS((const char *item, int indent_level, int is_operand, long user_data));
void UaSc_QHandleModify(WINDOW win, char *value);


void UaSc_QDoAdvanceTerm(WINDOW win)
{
	UAEP_DIALOG     *info;
	UAEP_DIALOG advInfo;

	info = (UAEP_DIALOG *)get_app_data(win);

	advInfo.cancelled = FALSE;
	advInfo.data.query.at_completeness = info->data.query.at_completeness;
	advInfo.data.query.at_position = info->data.query.at_position;
	advInfo.data.query.at_relation = info->data.query.at_relation;
	advInfo.data.query.at_structure = info->data.query.at_structure;
	advInfo.data.query.at_truncation = info->data.query.at_truncation;
	advInfo.data.query.at_use = info->data.query.at_use;
	get_title(get_ctl_window(win, SQ_DLG_VALUE),
	    advInfo.data.query.value,
	    sizeof(advInfo.data.query.value));


	create_res_dialog(WD_MODAL, 
		DLG_SEARCH_ADVQUERY, 
		EM_ALL, 
		dlg_109_eh, 
		(long)&advInfo);

	if (advInfo.cancelled == FALSE)
	{
		set_title(get_ctl_window(win, SQ_DLG_VALUE),
		    advInfo.data.query.value);
		info->data.query.at_completeness =
		    advInfo.data.query.at_completeness;
		info->data.query.at_position =
		    advInfo.data.query.at_position;
		info->data.query.at_relation =
		    advInfo.data.query.at_relation;
		info->data.query.at_structure =
		    advInfo.data.query.at_structure;
		info->data.query.at_truncation =
		    advInfo.data.query.at_truncation;
		info->data.query.at_use =
		    advInfo.data.query.at_use;
		info->data.query.class = RPN_LOCAL_DEFINE;
		info->data.query.className = "";
		UaSc_QDoneTerm(win);
	}
}

void UaSc_QHandleList(WINDOW win)
{
	int             index;
	long            listIndex;
	UAEP_DIALOG     *info;

	info = (UAEP_DIALOG *)get_app_data(win);
	/*
	 * They have selected a predefined type.
	 * Get the class information.
	 */
	index = win_list_get_sel_index(
	    get_ctl_window(win, SQ_DLG_FIELDNAME));

	info->data.query.class = RPN_PRE_DEFINE;
	info->data.query.classIndex = index;

	slist_elt(use_attribs, index, &listIndex);

	index = (int)listIndex;
	info->data.query.at_use = BasicDisplayList[index].use;
	info->data.query.at_relation = BasicDisplayList[index].relation;
	info->data.query.at_position = BasicDisplayList[index].position;
	info->data.query.at_structure = BasicDisplayList[index].structure;
	info->data.query.at_truncation = BasicDisplayList[index].truncation;
	info->data.query.at_completeness = BasicDisplayList[index].completeness;
	info->data.query.className = BasicDisplayList[index].className;
	Selected = TRUE;
}

void UaSc_QDoneTerm(WINDOW win)
{
	char                    at_value[BUFSIZ];
	WINDOW                  ctlwin;
	UAEP_DIALOG             *info = (UAEP_DIALOG *)get_app_data(win);
	RPN_AttribTerm  *new_attr;
	RPN_Operand     *new_oper;
	RPN_Structure   *new_term;

	/* Make sure that they've entered a value */
	if (!get_title(ctlwin = get_ctl_window(win, SQ_DLG_VALUE),
	    at_value, sizeof(at_value)) || at_value[0] == 0)
	{
		xvt_beep();
		set_front_window(ctlwin);
		return;
	}
	if ((info->data.query.class == RPN_PRE_DEFINE) && (Selected == FALSE))
	{
		UaSc_QHandleList(win);
	}

	if (ModifyMode == TRUE)
	{
		UaSc_QHandleModify(win, at_value);
		return;
	}

	new_attr = RPN_AttribTerm_New(
	    info->data.query.at_use,
	    info->data.query.at_relation,
	    info->data.query.at_position,
	    info->data.query.at_structure,
	    info->data.query.at_truncation,
	    info->data.query.at_completeness,
	    at_value,
	    info->data.query.class,
	    info->data.query.className,
	    info->data.query.classIndex);

	new_oper = RPN_Operand_NewAT(new_attr);
	RPN_AttribTerm_Free(new_attr);
	new_term = RPN_Structure_NewOp(new_oper);
	RPN_Operand_Free(new_oper);

	if (info->data.query.tree)
	{
		RPN_Structure   *parent, *sel;
		RPN_Expression  *ex;

		if (!(info->data.query.current_item_num))
		{
			/*
			 * The user hasn't selected an item from the tree
			 * so we'll add it to the end.
			 */
			info->data.query.current_item_num = win_list_count_all(
			    get_ctl_window(win, SQ_DLG_EXPR_TREE));
		}
		/* Attach the new term to the tree at the selected position */
		if (sel = RPN_Structure_TreeFind(info->data.query.tree,
		    info->data.query.current_item_num,
		    &parent))
		{
			RPN_Structure   *new_str;
			if (sel->rpn_struct_type != RPN_STRUCT_OPERAND)
			{
				/*
				 * The selected item is an
				 * operator (AND/OR/AND-NOT)  so add it
				 * to the next one down.
				 */
				parent = sel;
				sel = parent->v.expression->op2;
			}

			ex = RPN_Expression_New(sel,
			    info->data.query.operator,
			    new_term);
			new_str = RPN_Structure_NewEx(ex);
			RPN_Expression_Free(ex);
			if (parent)
			{
				if (parent->v.expression->op1 == sel)
				{
					parent->v.expression->op1 = new_str;
				}
				else
				{
					parent->v.expression->op2 = new_str;
				}
			}
			else
			{
				info->data.query.tree = new_str;
			}
			RPN_Structure_Free(sel);
		}
		else
		{
			xvt_error("Unable to locate selected subexpression!");
		}
		RPN_Structure_Free(new_term);
	}
	else
	{
		info->data.query.tree = new_term;
	}

	/*
	 * Allways make the last element selected.
	 */
	info->data.query.current_item_num =
	    RPN_Structure_Size(info->data.query.tree);
	show_query(win, info);
	UaSc_QClearButton(win);
}


void UaSc_QExprModify(WINDOW win, RPN_Operator oper, int delete)
{
	RPN_Structure   *parent, *sel;
	RPN_Expression	*deadExp;
	RPN_Structure   *deadStr;
	UAEP_DIALOG     *info;

	info = (UAEP_DIALOG *)get_app_data(win);

	/* First, find the selected sub-expression */

	if (!info->data.query.tree  ||  !info->data.query.current_item_num)
		return;


	if (!(sel = RPN_Structure_TreeFind(info->data.query.tree, info->data.query.current_item_num, &parent)))
		return;

	if (delete) {
		if (parent) 
		{
			deadExp = parent->v.expression;

			if (parent->v.expression->op1 == sel)
			{
				deadStr = parent->v.expression->op2;
				*parent = *parent->v.expression->op2;
			}
			else
			{
				deadStr = parent->v.expression->op1;
				*parent = *parent->v.expression->op1;
			}
			RPN_Structure_Free(sel);
			RescIR_Free(IR_FILE, (CHAR *)deadExp);
			RescIR_Free(IR_FILE, (CHAR *)deadStr);
		} 
		else 
		{
			/* We're deleting the entire tree! */
			RPN_Structure_Free(info->data.query.tree);
			info->data.query.tree = (RPN_Structure *)0;
		}
	} 
	else 
	{
		if (sel->rpn_struct_type == RPN_STRUCT_EXPR)
			sel->v.expression->operator = oper;
	}

	show_query(win, info);
}

void UaSc_QExprUpDown(WINDOW win, int direction)
{
	RPN_Structure   *parent1;
	RPN_Structure   *parent2;
	RPN_Structure	*sel1;
	RPN_Structure	*sel2;
	RPN_Structure	*tmp;
	UAEP_DIALOG     *info;
	SHORT		moveto;

	info = (UAEP_DIALOG *)get_app_data(win);

	/* First, find the selected sub-expression */

	moveto = info->data.query.current_item_num + 2;
	if (direction)
	{
		moveto -= 4;
	}

	if (!info->data.query.tree  ||  !info->data.query.current_item_num
	    || (info->data.query.current_item_num % 2 == 0)
	    || (moveto < 1) 
	    || (moveto > RPN_Structure_Size(info->data.query.tree)))
	{
		xvt_beep();
		return;
	}

	if (!(sel1 = RPN_Structure_TreeFind(
	    info->data.query.tree, 
	    info->data.query.current_item_num, 
	    &parent1)))
	{
		return;
	}
	if (!(sel2 = RPN_Structure_TreeFind(
	    info->data.query.tree, 
	    moveto,
	    &parent2)))
	{
		return;
	}

	if (parent1 && parent2) 
	{
		if (parent1->v.expression->op1 == sel1)
		{
			tmp = parent1->v.expression->op1;
			if (parent2->v.expression->op1 == sel2)
			{
				parent1->v.expression->op1 = sel2;
				parent2->v.expression->op1 = tmp;
			}
			else
			{
				parent1->v.expression->op1 = sel2;
				parent2->v.expression->op2 = tmp;
			}
		}
		else
		{
			tmp = parent1->v.expression->op2;
			if (parent2->v.expression->op1 == sel2)
			{
				parent1->v.expression->op2 = sel2;
				parent2->v.expression->op1 = tmp;
			}
			else
			{
				parent1->v.expression->op2 = sel2;
				parent2->v.expression->op2 = tmp;
			}
		}
	} 
	info->data.query.current_item_num  = moveto;

	show_query(win, info);
}


void UaSc_QExprSelected(WINDOW win, int dbl_click)
{
	int     is_expr;
	RPN_Structure   *parent, *sel;
	RPN_Operand     *op;
	RPN_AttribTerm  *term;
	UAEP_DIALOG *info;

	info = (UAEP_DIALOG *)get_app_data(win);
	info->data.query.current_item_num = win_list_get_sel_index(
	    get_ctl_window(win, SQ_DLG_EXPR_TREE)) + 1;

	if (!(info->data.query.tree))
	{
		/* There is nothing to select. */
		return;
	}
	/*
	 * Find the term at the selected position.
	 */
	if (!(sel = RPN_Structure_TreeFind(info->data.query.tree,
	    info->data.query.current_item_num,
	    &parent)))
	{
		/* Couldn't find the selected item. */
		return;
	}
	is_expr = sel->rpn_struct_type == RPN_STRUCT_EXPR;
	if (is_expr)
	{
		if (dbl_click)
		{
			if (++sel->v.expression->operator >=
			    RPNOP_NUM_OPERS)
			{
				sel->v.expression->operator =
				    (RPN_Operator)0;
			}
			show_query(win, info);
		}
		switch (sel->v.expression->operator)
		{
		case RPNOP_AND:
			UaSc_QOperSelected(win, SQ_DLG_RADIOBUTTON_OPER_AND);
			break;
		case RPNOP_OR:
			UaSc_QOperSelected(win, SQ_DLG_RADIOBUTTON_OPER_OR);
			break;
		case RPNOP_AND_NOT:
			UaSc_QOperSelected(win, SQ_DLG_RADIOBUTTON_OPER_AND_NOT);
			break;
		}
	}
	else if (dbl_click)
	{
		/*
		 * This is a term the user wants to modify.
		 */
		ModifyMode = TRUE;
		Selected = TRUE;

		op = sel->v.operand;
		term = op->v.rpn_op_at;
		info->data.query.at_use = term->use;
		info->data.query.at_relation = term->relation;


		info->data.query.at_position = term->position;
		info->data.query.at_structure = term->structure;
		info->data.query.at_truncation = term->truncation;
		info->data.query.at_completeness = term->completeness;
		info->data.query.class = term->class;
		info->data.query.classIndex = term->classIndex;
		info->data.query.className = term->className;
		set_title(get_ctl_window(win, SQ_DLG_VALUE), term->value);
		if (term->class == RPN_PRE_DEFINE)
		{
			win_list_set_sel(
			    get_ctl_window(win, SQ_DLG_FIELDNAME),
			    -1, FALSE);
			win_list_set_sel(
			    get_ctl_window(win, SQ_DLG_FIELDNAME),
			    term->classIndex, TRUE);
		}
		else
		{
			UaSc_QDoAdvanceTerm(win);
		}
	}
}


void UaSc_QHandleModify(WINDOW win, char *value)
{
	RPN_Structure   *parent, *sel;
	RPN_Operand     *op;
	RPN_AttribTerm  *term;
	UAEP_DIALOG     *info;

	info = (UAEP_DIALOG *)get_app_data(win);

	ModifyMode = FALSE;
	if (!(info->data.query.tree))
	{
		/* There is nothing to modify.  */
		return;
	}
	/*
	 * Find the term at the selected position.
	 */
	if (!(sel = RPN_Structure_TreeFind(info->data.query.tree,
	    info->data.query.current_item_num,
	    &parent)))
	{
		/* Couldn't find the selected item. */
		return;
	}
	term = RPN_AttribTerm_New(
	    info->data.query.at_use,
	    info->data.query.at_relation,
	    info->data.query.at_position,
	    info->data.query.at_structure,
	    info->data.query.at_truncation,
	    info->data.query.at_completeness,
	    value,
	    info->data.query.class,
	    info->data.query.className,
	    info->data.query.classIndex);

	op = sel->v.operand;
	RPN_AttribTerm_Free(op->v.rpn_op_at);
	op->v.rpn_op_at = term;
	show_query(win, info);
	UaSc_QClearButton(win);
}

void UaSc_QClearButton(WINDOW win)
{
	int cid;
	UAEP_DIALOG *info;

	info = (UAEP_DIALOG *)get_app_data(win);
	Selected = FALSE;
	ModifyMode = FALSE;
	set_title(get_ctl_window(win, SQ_DLG_VALUE), "");
	switch (info->data.query.operator)
	{
	case RPNOP_OR:
		cid = SQ_DLG_RADIOBUTTON_OPER_OR;
		break;
	case RPNOP_AND_NOT:
		cid = SQ_DLG_RADIOBUTTON_OPER_AND_NOT;
		break;
	case RPNOP_AND:
	default:
		cid = SQ_DLG_RADIOBUTTON_OPER_AND;
		break;
	}
	UaSc_QOperSelected(win, cid);

	/*
	 * Reset the internal settings.
	 */
	info->data.query.at_completeness = RPN_AT_CPL_UNSPECIFIED;
	info->data.query.at_position = RPN_AT_POS_UNSPECIFIED;
	info->data.query.at_relation = RPN_AT_REL_EQUAL;
	info->data.query.at_structure = RPN_AT_STR_UNSPECIFIED;
	info->data.query.at_truncation = RPN_AT_TRN_UNSPECIFIED;
	info->data.query.at_use = RPN_AT_USE_AUTHOR;
	info->data.query.class = RPN_PRE_DEFINE;
	info->data.query.className = "";
}

/*
 *      init_dialog()
 *
 *      Does the creation-time per-dialog initialization.
 *
 */
void UaSc_QInitDialog(WINDOW win)
{
	UAEP_DIALOG     *info = (UAEP_DIALOG *)get_app_data(win);
	SHORT           i;      /* Counter */

	/* One-time initialization */
	if (!use_attribs)
	{
		if  (!(use_attribs = slist_new()))
		{
			xvt_error("Not enough memory!");
			return;
		}
		for (i = 0;
		    i < (sizeof(BasicDisplayList)/sizeof(BasicDisplayList[0])); i++)
		{
			 (void)slist_add_sorted(use_attribs,
			     (STR)BasicDisplayList[i].value,
			     (long)i,
			     FALSE,
			     FALSE);

		}
	}

	enable_window(get_ctl_window(win, SQ_DLG_PUSHBUTTON_DELETE),
	    TRUE);

	win_list_add(get_ctl_window(win, SQ_DLG_FIELDNAME),
	     -1, (STR)use_attribs);
	/*
	 * Unselect the entire list.
	 */
	win_list_set_sel(get_ctl_window(win, SQ_DLG_FIELDNAME),
	    -1, FALSE);
	/*
	 * Select the first element.
	 */
	win_list_set_sel(get_ctl_window(win, SQ_DLG_FIELDNAME),
	    2,TRUE);

	/* Set default selections for radio-buttons etc. */
	show_query(win, info);
	UaSc_QClearButton(win);
	set_front_window(get_ctl_window(win, SQ_DLG_VALUE));
}


void UaSc_QOperSelected(WINDOW win, short cid)
{
	UAEP_DIALOG *info;
	WINDOW  controls[3];
	WINDOW  *ctl_ptr = controls;

	info = (UAEP_DIALOG *)get_app_data(win);
	switch (cid) {
	case SQ_DLG_RADIOBUTTON_OPER_AND:
		info->data.query.operator = RPNOP_AND;
		goto operator;
	case SQ_DLG_RADIOBUTTON_OPER_OR:
		info->data.query.operator = RPNOP_OR;
		goto operator;
	case SQ_DLG_RADIOBUTTON_OPER_AND_NOT:
		info->data.query.operator = RPNOP_AND_NOT;
	operator:
		*ctl_ptr++ = get_ctl_window(win, SQ_DLG_RADIOBUTTON_OPER_AND);
		*ctl_ptr++ = get_ctl_window(win, SQ_DLG_RADIOBUTTON_OPER_OR);
		*ctl_ptr++ = get_ctl_window(win, SQ_DLG_RADIOBUTTON_OPER_AND_NOT);
		win_check_radio_button(get_ctl_window(win, cid), controls, ctl_ptr - controls);
		break;
	}
}


static void show_query(WINDOW win, UAEP_DIALOG *info)
{
	WINDOW  ctlwin;
	char    *query;

	ctlwin = get_ctl_window(win, SQ_DLG_EXPR_TREE);

	if (!info->data.query.tree) {
		set_title(get_ctl_window(win, SQ_DLG_EXPR_TEXT), "");
		win_list_clear(ctlwin);
		return;
	}

	if (query = RPN_Structure_Print(info->data.query.tree)) {
		set_title(get_ctl_window(win, SQ_DLG_EXPR_TEXT), query);
		RescIR_Free(IR_FILE, query);
	}

	win_list_suspend(ctlwin);
	win_list_clear(ctlwin);
	RPN_Structure_TreePrint(info->data.query.tree, tree_to_list, ctlwin);
	win_list_resume(ctlwin);
	win_list_set_sel(ctlwin, 
	    max(info->data.query.current_item_num - 1, 0), 
	    TRUE);
	UaSc_QExprSelected(win, FALSE);
}


static int
#ifdef XVT_CC_PROTO
tree_to_list(const char *item, int indent_level, int is_operand, long user_data)
#else
tree_to_list(item, indent_level, is_operand, user_data)
char 	*item;
int	indent_level;
int	is_operand;
long	user_data;
#endif
{
	char	buf[BUFSIZ];
	
	sprintf(buf, "%*s%s", indent_level * 3, "", item);

	return !win_list_add((WINDOW)user_data, -1, buf);
}
