
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <ua.h>
#include <resc.h>
#include <resclist.h>
#include <debug.h>
#include <uaep.h>


void UaSc_DbInitDbInfoStruct(UASC_DB_INFO_STRUCT *db)
{
	IR_USER_TARGET_INFO	*userInfo;
	IR_TARGET_INFO		*tgInfo;
	
	RESC_ASN_MALLOCED(db);
	RESC_LIST_INIT(&db->tgInfo.dbInfo);
	RESC_ASN_NO_MALLOC(&db->tgInfo.dbInfo);
	RESC_ASN_NO_MALLOC(&db->tgInfo);
	RESC_ASN_NO_MALLOC(&db->userTgInfo);

	userInfo = &db->userTgInfo;
	tgInfo = &db->tgInfo;
	db->assocId = IR_UNKNOWN_ID_TYPE;

	userInfo->targetId = IR_UNKNOWN_ID_TYPE;
	RESC_ASN_NO_MALLOC(&userInfo->targetName);
	RESC_ASN_INIT_VS(&userInfo->targetName);
	userInfo->defSmallSetUpperBound = 0;
	userInfo->defLargeSetLowerBound = 0;
	userInfo->defPresentNumber = 0;
	RESC_ASN_NO_MALLOC(&userInfo->defPreferedRecSyntax);
	RESC_ASN_INIT_VS(&userInfo->defPreferedRecSyntax);
	RESC_ASN_NO_MALLOC(&userInfo->defElementSetName);
	RESC_ASN_INIT_VS(&userInfo->defElementSetName);
	userInfo->defResultSetNamingSupported = FALSE;
	userInfo->defResultSetNamingSupported = FALSE;
	RESC_ASN_NO_MALLOC(&userInfo->displayFormats);
	RESC_ASN_INIT_VS(&userInfo->displayFormats);

	tgInfo->targetId = IR_UNKNOWN_ID_TYPE;
	RESC_ASN_NO_MALLOC(&tgInfo->targetName);
	RESC_ASN_INIT_VS(&tgInfo->targetName);
	RESC_ASN_NO_MALLOC(&tgInfo->description);
	RESC_ASN_INIT_VS(&tgInfo->description);
	RESC_ASN_NO_MALLOC(&tgInfo->options);
	RESC_ASN_INIT_BS(&tgInfo->options);
	RESC_ASN_NO_MALLOC(&tgInfo->contactInfo);
	RESC_ASN_INIT_VS(&tgInfo->contactInfo);
	tgInfo->resultSetNamingSupported = FALSE;
	tgInfo->elementSetNamingSupported = FALSE;
	tgInfo->maxNumResultSets = 0;
	tgInfo->hoursOfOperation = 0;
	RESC_ASN_NO_MALLOC(&tgInfo->chargingPolicy);
	RESC_ASN_INIT_VS(&tgInfo->chargingPolicy);
	RESC_ASN_NO_MALLOC(&tgInfo->dbInfo);
	RESC_LIST_INIT(&tgInfo->dbInfo);
	RESC_ASN_NO_MALLOC(&tgInfo->idAuthentication);
	RESC_ASN_INIT_VS(&tgInfo->idAuthentication);
	
}

void UaSc_DbDestroyDbInfoStruct(CHAR *src, void *dbInfo)
{
	UASC_DB_INFO_STRUCT	*db;

	if (dbInfo == NULL)
	{
		return;
	}
	db = (UASC_DB_INFO_STRUCT *)dbInfo;

	RescASN_DestroyVS(src, &db->userTgInfo.targetName);
	RescASN_DestroyVS(src, &db->userTgInfo.defPreferedRecSyntax);
	RescASN_DestroyVS(src, &db->userTgInfo.defElementSetName);
	RescASN_DestroyVS(src, &db->userTgInfo.displayFormats);

	RescASN_DestroyVS(src, &db->tgInfo.targetName);
	RescASN_DestroyVS(src, &db->tgInfo.description);
	RescASN_DestroyBS(src, &db->tgInfo.options);
	RescASN_DestroyVS(src, &db->tgInfo.contactInfo);
	RescASN_DestroyVS(src, &db->tgInfo.chargingPolicy);
	RescList_Destroy(src, &db->tgInfo.dbInfo);
	RescASN_DestroyVS(src, &db->tgInfo.idAuthentication);
}

void UaSc_DbFillTargetInfo( IR_TARGET_ID targetId,
    UASC_DB_INFO_STRUCT *data)
{
	IRO_USER_INFO_REQ   uReq;
	IRO_USER_INFO_REPLY uReply;
	IR_USER_TARGET_INFO *uTarget;
	SHORT		    index;
	IRO_TARGET_INFO_REPLY	targetReply;
	IR_TARGET_INFO		*tgInfo;

	/*
	 * Find target info known by this user.
	 */
	uReq.myUserId = UaEp_InfoUserId();
	RESC_ASN_NO_MALLOC(&uReq);
	RESC_ASN_NO_MALLOC(&uReply);

	if (IRO_UserInfo(IRO_UI_TARGETS, &uReq, &uReply) != IR_SUCCESS)
	{
		CS_LOG_PRINTS(IR_FILE, "Get user info failed\n");
		return;
	}
	
	/* Check for target info */
	if (RESC_LIST_SIZE(uReply.targetInfo) == 0)
	{
		CS_LOG_PRINTS(IR_FILE, "No user target info available for this user\n");
		return;
	}

	/* Find target info matching input target Id */
	RESC_LIST_SET_FIRST(uReply.targetInfo);
	for (index = 0; RESC_LIST_SIZE(uReply.targetInfo); index++)
	{
		uTarget = RESC_LIST_GET(uReply.targetInfo);
		if (uTarget->targetId == targetId)
		{
			/* This is the one */
			break;
		}
		else
		{
			RESC_LIST_SET_NEXT(uReply.targetInfo);
		}
	}

	if (uTarget->targetId != targetId)
	{
		/* No matching targetId found */
		CS_LOG_PRINTV(IR_FILE, "Target not found. %d\n", targetId);
		return;
	}

	/*
	 *  Copy the user target information from the list element to 
	 *  the UASC_DB_INFO_STRUCT element.
	 */
	data->userTgInfo.targetId = uTarget->targetId;
	RescASN_CopyVS(IR_FILE, &data->userTgInfo.targetName,
	    &uTarget->targetName);
	RescASN_CopyVS(IR_FILE, &data->userTgInfo.userAuthentication,
	    &uTarget->userAuthentication);
	data->userTgInfo.defSmallSetUpperBound = uTarget->defSmallSetUpperBound;
	data->userTgInfo.defLargeSetLowerBound = uTarget->defLargeSetLowerBound;
	data->userTgInfo.defPresentNumber = uTarget->defPresentNumber;
	RescASN_CopyOI(IR_FILE, &data->userTgInfo.defPreferedRecSyntax,
	    &uTarget->defPreferedRecSyntax);
	RescASN_CopyVS(IR_FILE, &data->userTgInfo.defElementSetName,
	    &uTarget->defElementSetName);
	data->userTgInfo.defResultSetNamingSupported =
	    uTarget->defResultSetNamingSupported;
	RescASN_CopyVS(IR_FILE, &data->userTgInfo.displayFormats,
	    &uTarget->displayFormats);


	/* Now Get the target information call IRO_TargetInfo */
	if (IRO_TargetInfo(UaEp_InfoUserId(), IRO_TRGT_INFO_GEN,
	    targetId, &targetReply) != IR_SUCCESS)
	{
		CS_LOG_PRINTV(IR_FILE, 
		    "Failed getting target information for target Id %d\n",
		    targetId);
	} 
	/* Copy target info to local structure using a pointer */
	tgInfo  = targetReply.targetGenInfo;
	
	data->tgInfo.targetId = tgInfo->targetId;
	RescASN_CopyVS(IR_FILE, &data->tgInfo.targetName,
	    &tgInfo->targetName);
	RescASN_CopyVS(IR_FILE, &data->tgInfo.description,
	    &tgInfo->description);
	RescASN_CopyBS(IR_FILE, &data->tgInfo.options,
	    &tgInfo->options);
	RescASN_CopyVS(IR_FILE, &data->tgInfo.contactInfo,
	    &tgInfo->contactInfo);
	data->tgInfo.resultSetNamingSupported =
	    tgInfo->resultSetNamingSupported;
	data->tgInfo.elementSetNamingSupported =
	    tgInfo->elementSetNamingSupported;
	data->tgInfo.maxNumResultSets =
	    tgInfo->maxNumResultSets;
	data->tgInfo.hoursOfOperation=
	    tgInfo->hoursOfOperation;
	RescASN_CopyVS(IR_FILE, &data->tgInfo.chargingPolicy,
	    &tgInfo->chargingPolicy);
	RescASN_CopyVS(IR_FILE, &data->tgInfo.idAuthentication,
	    &tgInfo->idAuthentication);


	/* Copy the database information */
	RescList_Copy(IR_FILE, &data->tgInfo.dbInfo,
	    &tgInfo->dbInfo);

CS_LOG_PRINTS(IR_FILE, "Got target info\n");

}

UASC_DB_INFO_STRUCT *UaSc_DbCreateDbInfoStruct(CHAR *src)
{
	UASC_DB_INFO_STRUCT *new;

	new = (UASC_DB_INFO_STRUCT *)RescIR_Zmalloc(IR_FILE, 
	    sizeof(UASC_DB_INFO_STRUCT));

	if (new == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Malloc error\n");
		return(new);
	}
	RESC_ASN_IS_MALLOCED(new);
	return(new);
}

/*
 *
 * DIALOG DATABASE OPTIONS support code.
 * for displaying and obtaining selected record syntax and 
 * record composition types.
 *
 */

typedef struct
{
	CHAR	*oid;
	CHAR	*disp;
	BOOLEAN	used;
} REC_SYNTAX_TO_OID;

static REC_SYNTAX_TO_OID	DoDlg_DispSyntaxList[]={
	{
		OID_REC_SYN_UNIMARC ,
		"UNIMARC",
		0
	},
	{
		OID_REC_SYN_INTERMARC,
		"INTERMARC",
		0
	},
	{
		OID_REC_SYN_CCF,
		"CCF",
		0
	},
	{
		OID_REC_SYN_USMARC,
		"USMARC",
		0
	},
	{
		OID_REC_SYN_UKMARC,
		"UKMARC",
		0
	},
	{
		OID_REC_SYN_NORMARC,
		"NORMARC",
		0
	},
	{
		OID_REC_SYN_LIBRISMARC,
		"LIBRISMARC",
		0
	},
	{
		OID_REC_SYN_DANMARC,
		"DANMARC",
		0
	},
	{
		OID_REC_SYN_FINMARC,
		"FINMARC",
		0
	},
	{
		OID_REC_SYN_MAB,
		"MAB",
		0
	},
	{
		OID_REC_SYN_CANMARC,
		"CANMARC",
		0
	},
	{ 
		OID_REC_SYN_SBN ,
		"SBN",
		0
	},
	{
		OID_REC_SYN_PICAMARC,
		"PICAMARC",
		0
	},
	{
		(CHAR *)NULL,
		(CHAR *)NULL,
		0
	}
};

		

/* Use list of databases to determine the record syntax to display */
void UaSc_DispSyntaxList(IR_LIST *dbList, SLIST dispList)
{
	IR_DATABASE_INFO	*db;
	SHORT			index;
	SHORT			index1;
	SHORT			j;
	SHORT			len;
	OBJECT_IDENTIFIER	*oi;


	if (RESC_LIST_SIZE(dbList) == 0)
	{
		return;
	}

	/* Reset the display list */
	index = 0;
	while (DoDlg_DispSyntaxList[index].oid != (CHAR *)NULL)
	{
		DoDlg_DispSyntaxList[index].used = FALSE;
		index++;
	}

	RESC_LIST_SET_FIRST(dbList);
	for (index = 0; index < RESC_LIST_SIZE(dbList); index++)
	{
		db = RESC_LIST_GET(dbList);

		/* Scan the object identifier list */
		RESC_LIST_SET_FIRST(&db->recordSyntaxList);
		for (index1 = 0; 
		    index1 < RESC_LIST_SIZE(&db->recordSyntaxList);
		    index1++)
		{
			oi = RESC_LIST_GET(&db->recordSyntaxList);
			RescUT_DebugVS(IR_FILE, oi);

			j = 0;
			while (DoDlg_DispSyntaxList[j].oid != (CHAR *)NULL)
			{
				len = strlen(DoDlg_DispSyntaxList[j].oid);
				if (len == (SHORT)oi->len)
				{
					if (strncmp(oi->sPtr,
				    	    DoDlg_DispSyntaxList[j].oid,
					    len) == 0)
					{
						DoDlg_DispSyntaxList[j].used 
						    = TRUE;
						break;
					}
				}
				j++;
			}

			RESC_LIST_SET_NEXT(&db->recordSyntaxList);
		}
		
		RESC_LIST_SET_NEXT(dbList);
	}

	/* 
	 * For each entry in the disp syntax list marked as used
	 * add it to the display list.
	 */
	j = 0;
	while (DoDlg_DispSyntaxList[j].oid != (CHAR *)NULL)
	{
		if (DoDlg_DispSyntaxList[j].used == TRUE)
		{
			slist_add_sorted(dispList,
			    (STR)DoDlg_DispSyntaxList[j].disp,
			    (long)j, TRUE, FALSE);

		}
		j++;
	}
}

/* Use list of databases to determine the record syntax to display */
void UaSc_DispElemSetList(IR_LIST *dbList , SLIST dispList)
{
	IR_DATABASE_INFO	*db;
	SHORT			index;
	SHORT			index1;
	VISIBLE_STRING		*vs;
	CHAR			tmpBuf[50];

	
	if (RESC_LIST_SIZE(dbList) == 0)
	{
		return;
	}

	RESC_LIST_SET_FIRST(dbList);
	for (index = 0; index < RESC_LIST_SIZE(dbList); index++)
	{
		db = RESC_LIST_GET(dbList);

		/* Scan the object identifier list */
		RESC_LIST_SET_FIRST(&db->elementSetNames);
	 	for (index1 = 0; 
		    index1 < RESC_LIST_SIZE(&db->elementSetNames);
		    index1++)
		{
			vs = RESC_LIST_GET(&db->elementSetNames);
			VSToStrNullTerm(tmpBuf, vs, 50);
			slist_add_sorted(dispList,
			    tmpBuf, 
			    (long)NULL,
			    TRUE,
			    FALSE);
			RESC_LIST_SET_NEXT(&db->elementSetNames);
		}
		RESC_LIST_SET_NEXT(dbList);
	}
}

void UaSc_GetElemSetNameFromList(WINDOW win, SLIST list, 
	VISIBLE_STRING *dest)
{
	CHAR	tmpBuf[IR_LEN_100];
	/* Get the selected item from the list. */
	if (win_list_get_first_sel(win, tmpBuf, IR_LEN_100) == TRUE)
	{
		RescASN_AddVS(IR_FILE, dest, tmpBuf, strlen(tmpBuf));
	}
	else
	{
		RESC_ASN_INIT_VS(dest);
	}
}

void UaSc_GetSyntaxFromList(WINDOW win, SLIST list, 
	OBJECT_IDENTIFIER *dest)
{
	long	data;
	SHORT	j;

	/* Get the selected item from the list. */
	if (EpDialog_GetSelListData(win, list, &data) == IR_FAIL)
	{
		/* No item selected */
		RESC_ASN_INIT_OI(dest);
		return;
	}
	else
	{
		j = (SHORT)data;
		if (j <
		    (sizeof(DoDlg_DispSyntaxList)/sizeof(REC_SYNTAX_TO_OID)))
		{
			
			
			RescASN_AddOI(IR_FILE, dest, 
			    DoDlg_DispSyntaxList[j].oid,
			    strlen(DoDlg_DispSyntaxList[j].oid));
		}
	}
}
